@@ -1,5 +1,5 @@
 /*  Functions from hack's utils library.
-    Copyright (C) 1989, 1990, 1991, 1998, 1999
+    Copyright (C) 1989, 1990, 1991, 1998, 1999, 2003
     Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -20,14 +20,6 @@
 
 #include <stdio.h>
 
-#if ENABLE_NLS
-# include <libintl.h>
-#else
-# define gettext(msgid) (msgid)
-# define ngettext(sing,plur,n) ((n)==1 ? (sing) : (plur))
-#endif
-#define _(String) gettext(String)
-
 #include <errno.h>
 #ifndef errno
   extern int errno;
@@ -43,12 +35,24 @@
 # include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
-#include "basicdefs.h"
 #include "utils.h"
 
 const char *myname;
 
-void do_ck_fclose P_((FILE *stream));
+/* Store information about files opened with ck_fopen
+   so that error messages from ck_fread, ck_fwrite, etc. can print the
+   name of the file that had the error */
+
+struct open_file
+  {
+    FILE *fp;
+    char *name;
+    struct open_file *link;
+    unsigned temp : 1;
+  };
+
+static struct open_file *open_files = NULL;
+static void do_ck_fclose P_((FILE *fp));
 
 /* Print an error message and exit */
 #if !defined __STDC__ || !(__STDC__-0)
@@ -80,32 +84,36 @@ panic(const char *str, ...)
 #endif /* HAVE_VFPRINTF */
   va_end(iggy);
   putc('\n', stderr);
+
+  /* Unlink the temporary files.  */
+  while (open_files)
+    {
+      if (open_files->temp)
+	{
+	  int fd = fileno (open_files->fp);
+	  fclose (open_files->fp);
+	  errno = 0;
+	  unlink (open_files->name);
+          if (errno != 0)
+            fprintf (stderr, _("cannot remove %s: %s"), open_files->name, strerror (errno));
+	}
+
+      open_files = open_files->link;
+    }
+
   exit(4);
 }
 
 
-/* Store information about files opened with ck_fopen
-   so that error messages from ck_fread, ck_fwrite, etc. can print the
-   name of the file that had the error */
-
-struct id
-  {
-    FILE *fp;
-    char *name;
-    struct id *link;
-  };
-
-static struct id *utils_id_s = NULL;
-
-/* Internal routine to get a filename from utils_id_s */
+/* Internal routine to get a filename from open_files */
 static const char *utils_fp_name P_((FILE *fp));
 static const char *
 utils_fp_name(fp)
   FILE *fp;
 {
-  struct id *p;
+  struct open_file *p;
 
-  for (p=utils_id_s; p; p=p->link)
+  for (p=open_files; p; p=p->link)
     if (p->fp == fp)
       return p->name;
   if (fp == stdin)
@@ -120,16 +128,24 @@ utils_fp_name(fp)
 
 /* Panic on failing fopen */
 FILE *
-ck_fopen(name, mode)
+ck_fopen(name, mode, fail)
   const char *name;
   const char *mode;
+  bool fail;
 {
   FILE *fp;
-  struct id *p;
+  struct open_file *p;
 
-  if ( ! (fp = fopen(name, mode)) )
-    panic(_("Couldn't open file %s: %s"), name, strerror(errno));
-  for (p=utils_id_s; p; p=p->link)
+  fp = fopen (name, mode);
+  if (!fp)
+    {
+      if (fail)
+        panic(_("couldn't open file %s: %s"), name, strerror(errno));
+
+      return NULL;
+    }
+
+  for (p=open_files; p; p=p->link)
     {
       if (fp == p->fp)
 	{
@@ -139,12 +155,55 @@ ck_fopen(name, mode)
     }
   if (!p)
     {
-      p = MALLOC(1, struct id);
-      p->link = utils_id_s;
-      utils_id_s = p;
+      p = MALLOC(1, struct open_file);
+      p->link = open_files;
+      open_files = p;
     }
   p->name = ck_strdup(name);
   p->fp = fp;
+  p->temp = false;
+  return fp;
+}
+
+FILE *
+ck_mkstemp (p_filename, tmpdir, base)
+  char **p_filename;
+  char *base, *tmpdir;
+{
+  char *template;
+  FILE *fp;
+  int fd;
+  struct open_file *p;
+
+  if (tmpdir == NULL)
+    tmpdir = getenv("TMPDIR");
+  if (tmpdir == NULL)
+    {
+      tmpdir = getenv("TMP");
+      if (tmpdir == NULL)
+#ifdef P_tmpdir
+	tmpdir = P_tmpdir;
+#else
+	tmpdir = "/tmp";
+#endif
+    }
+
+  template = xmalloc (strlen (tmpdir) + strlen (base) + 8);
+  sprintf (template, "%s/%sXXXXXX", tmpdir, base);
+
+  fd = mkstemp (template);
+  if (fd == -1)
+    panic(_("couldn't open temporary file %s: %s"), template, strerror(errno));
+
+  *p_filename = template;
+  fp = fdopen (fd, "w");
+
+  p = MALLOC(1, struct open_file);
+  p->name = ck_strdup (template);
+  p->fp = fp;
+  p->temp = true;
+  p->link = open_files;
+  open_files = p;
   return fp;
 }
 
@@ -178,6 +237,22 @@ ck_fread(ptr, size, nmemb, stream)
   return nmemb;
 }
 
+size_t
+ck_getline(text, buflen, stream)
+  char **text;
+  size_t *buflen;
+  FILE *stream;
+{
+  int result;
+  if (!ferror (stream))
+    result = getline (text, buflen, stream);
+
+  if (ferror (stream))
+    panic (_("read error on %s: %s"), utils_fp_name(stream), strerror(errno));
+
+  return result;
+}
+
 /* Panic on failing fflush */
 void
 ck_fflush(stream)
@@ -185,7 +260,7 @@ ck_fflush(stream)
 {
   clearerr(stream);
   if (fflush(stream) == EOF && errno != EBADF)
-    panic("Couldn't flush %s: %s", utils_fp_name(stream), strerror(errno));
+    panic("couldn't flush %s: %s", utils_fp_name(stream), strerror(errno));
 }
 
 /* Panic on failing fclose */
@@ -193,12 +268,12 @@ void
 ck_fclose(stream)
   FILE *stream;
 {
-  struct id r;
-  struct id *prev;
-  struct id *cur;
+  struct open_file r;
+  struct open_file *prev;
+  struct open_file *cur;
 
   /* a NULL stream means to close all files */
-  r.link = utils_id_s;
+  r.link = open_files;
   prev = &r;
   while ( (cur = prev->link) )
     {
@@ -213,7 +288,7 @@ ck_fclose(stream)
 	prev = cur;
     }
 
-  utils_id_s = r.link;
+  open_files = r.link;
 
   /* Also care about stdout, because if it is redirected the
      last output operations might fail and it is important
@@ -225,43 +300,49 @@ ck_fclose(stream)
     }
 }
 
-/* Close a single file and update a count of closed files. */
+/* Close a single file. */
 void
-do_ck_fclose(stream)
-  FILE *stream;
+do_ck_fclose(fp)
+  FILE *fp;
 {
-  ck_fflush(stream);
-  clearerr(stream);
-  if (fclose(stream) == EOF)
-    panic("Couldn't close %s: %s", utils_fp_name(stream), strerror(errno));
+  int fd;
+  ck_fflush(fp);
+  clearerr(fp);
+
+  /* We want to execute both arms, so use | not ||.  */
+  if (fclose(fp) == EOF)
+    panic("couldn't close %s: %s", utils_fp_name(fp), strerror(errno));
 }
-
-
 
-char *
-temp_file_template(tmpdir, program)
-  const char *tmpdir;    
-  char *program;
+
+/* Panic on failing rename */
+void
+ck_rename (from, to, unlink_if_fail)
+  const char *from, *to;
+  const char *unlink_if_fail;
 {
-  char *template;
-  if (tmpdir == NULL)
-    tmpdir = getenv("TMPDIR");
-  if (tmpdir == NULL)
+  int rd = rename (from, to);
+  if (rd != -1)
+    return;
+
+  if (unlink_if_fail)
     {
-      tmpdir = getenv("TMP");
-      if (tmpdir == NULL)
-#ifdef P_tmpdir
-	tmpdir = P_tmpdir;
-#else
-	tmpdir = "/tmp";
-#endif
+      int save_errno = errno;
+      errno = 0;
+      unlink (unlink_if_fail);
+
+      /* Failure to remove the temporary file is more severe, so trigger it first.  */
+      if (errno != 0)
+        panic (_("cannot remove %s: %s"), unlink_if_fail, strerror (errno));
+
+      errno = save_errno;
     }
 
-  template = xmalloc (strlen (tmpdir) + strlen (program) + 8);
-  sprintf (template, "%s/%sXXXXXX", tmpdir, program);
-  return (template);
+  panic (_("cannot rename %s: %s"), from, strerror (errno));
 }
 
+
+
 
 /* Panic on failing malloc */
 VOID *
@@ -270,7 +351,7 @@ ck_malloc(size)
 {
   VOID *ret = calloc(1, size ? size : 1);
   if (!ret)
-    panic("Couldn't allocate memory");
+    panic("couldn't allocate memory");
   return ret;
 }
 
@@ -299,7 +380,7 @@ ck_realloc(ptr, size)
     return ck_malloc(size);
   ret = realloc(ptr, size);
   if (!ret)
-    panic("Couldn't re-allocate memory");
+    panic("couldn't re-allocate memory");
   return ret;
 }
 
@@ -391,19 +472,21 @@ resize_buffer(b, newlen)
   b->b = try;
 }
 
-void
+char *
 add_buffer(b, p, n)
   struct buffer *b;
   const char *p;
   size_t n;
 {
+  char *result;
   if (b->allocated - b->length < n)
     resize_buffer(b, b->length+n);
-  memcpy(b->b + b->length, p, n);
+  result = memcpy(b->b + b->length, p, n);
   b->length += n;
+  return result;
 }
 
-void
+char *
 add1_buffer(b, c)
   struct buffer *b;
   int c;
@@ -416,10 +499,15 @@ add1_buffer(b, c)
    */
   if (c != EOF)
     {
+      char *result;
       if (b->allocated - b->length < 1)
 	resize_buffer(b, b->length+1);
-      b->b[b->length++] = c;
+      result = b->b + b->length++;
+      *result = c;
+      return result;
     }
+
+  return NULL;
 }
 
 void
