@@ -63,6 +63,7 @@ typedef struct eptrblock
   {
     struct eptrblock *prev;
     const uschar *saved_eptr;
+    int flags;
   }
 eptrblock;
 
@@ -70,6 +71,7 @@ eptrblock;
 
 #define match_condassert   0x01	/* Called to check a condition assertion */
 #define match_isgroup      0x02	/* Set if start of bracketed group */
+#define match_isbrazero    0x04	/* Set if matching optional bracketed group */
 
 
 /*************************************************
@@ -134,7 +136,7 @@ pchars (p, length, is_subject, md)
 static BOOL
 match_ref (offset, eptr, length, md, ims)
      int offset;
-     register const uschar *eptr;
+     const uschar *eptr;
      int length;
      match_data *md;
      unsigned long int ims;
@@ -154,13 +156,11 @@ match_ref (offset, eptr, length, md, ims)
   printf ("\n");
 #endif
 
-/* Always fail if not enough characters left */
-
+  /* Always fail if not enough characters left */
   if (length > md->end_subject - eptr)
     return FALSE;
 
-/* Separate the caselesss case for speed */
-
+  /* Separate the caselesss case for speed */
   if ((ims & PCRE_CASELESS) != 0)
     {
       while (length-- > 0)
@@ -198,15 +198,16 @@ match_ref (offset, eptr, length, md, ims)
    brackets - for testing for empty matches
    flags       can contain
    match_condassert - this is an assertion condition
-   match_isgroup - this is the start of a bracketed group
+   match_isgroup    - this is the start of a bracketed group
+   match_isbrazero  - this is after a BRAZERO
 
    Returns:       TRUE if matched
  */
 
 static BOOL
 match (eptr, ecode, offset_top, md, ims, eptrb, flags)
-     register const uschar *eptr;
-     register const uschar *ecode;
+     const uschar *eptr;
+     const uschar *ecode;
      int offset_top;
      match_data *md;
      unsigned long int ims;
@@ -217,26 +218,25 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
   const uschar *data;
   eptrblock newptrb;
 
-/* At the start of a bracketed group, add the current subject pointer to the
-   stack of such pointers, to be re-instated at the end of the group when we hit
-   the closing ket. When match() is called in other circumstances, we don't add to
-   the stack. */
-
+  /* At the start of a bracketed group, add the current subject pointer to the
+     stack of such pointers, to be re-instated at the end of the group when we hit
+     the closing ket. When match() is called in other circumstances, we don't add to
+     the stack. */
   if ((flags & match_isgroup) != 0)
     {
       newptrb.prev = eptrb;
       newptrb.saved_eptr = eptr;
+      newptrb.flags = flags;
       eptrb = &newptrb;
     }
 
-/* Now start processing the operations. */
-
+  /* Now start processing the operations. */
   for (;;)
     {
       int op = (int) *ecode;
       int min, max, ctype;
-      register int i;
-      register int c;
+      int i;
+      int c;
       int kind = 0;
 
       /* Opening capturing bracket. If there is space in the offset vector, save
@@ -266,7 +266,6 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 
 	  offset = number << 1;
 
-
 #ifdef DEBUG
 	  printf ("start bracket %d subject=", number);
 	  pchars (eptr, 16, TRUE, md);
@@ -588,7 +587,8 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 	case OP_BRAZERO:
 	  {
 	    const uschar *next = ecode + 1;
-	    if (match (eptr, next, offset_top, md, ims, eptrb, match_isgroup))
+	    if (match (eptr, next, offset_top, md, ims, eptrb,
+		       match_isgroup | match_isbrazero))
 	      return TRUE;
 	    do
 	      next += (next[1] << 8) + next[2];
@@ -650,21 +650,26 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 
 		offset = number << 1;
 #ifdef DEBUG
-		printf ("end bracket %d", number);
-		printf ("\n");
+		printf ("end bracket %d\n", number);
 #endif
 
-		if (number > 0)
+		/* Void matches (eptr == saved_eptr) are reported if they are the
+		   first ones (md->offset_vector[offset] is still -1) or if the
+		   match is not optional (the match_isbrazero flag is not active).  */
+		if (number > 0
+		    && (eptr > saved_eptr
+			|| offset_top <= offset
+			|| md->offset_vector[offset] == -1
+			|| !(eptrb->flags & match_isbrazero)))
 		  {
 		    if (offset >= md->offset_max)
 		      md->offset_overflow = TRUE;
-		    else
 		      {
+			if (offset_top <= offset)
+			  offset_top = offset + 2;
 			md->offset_vector[offset] =
 			  md->offset_vector[md->offset_end - number];
 			md->offset_vector[offset + 1] = eptr - md->start_subject;
-			if (offset_top <= offset)
-			  offset_top = offset + 2;
 		      }
 		  }
 	      }
@@ -728,6 +733,14 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 	  ecode++;
 	  continue;
 
+	  /* No implicit advancing assertion */
+
+	case OP_ANCHOR_MATCH:
+	  if (eptr != md->first_start)
+	    return FALSE;
+	  ecode++;
+	  continue;
+
 	  /* Assert before internal newline if multiline, or before a terminating
 	     newline unless endonly is set, else end of subject unless noteol is set. */
 
@@ -921,9 +934,7 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 	    if (length == 0)
 	      continue;
 
-	    /* First, ensure the minimum number of matches are present. We get back
-	       the length of the reference string explicitly rather than passing the
-	       address of eptr, so that eptr can be a register variable. */
+	    /* First, ensure the minimum number of matches are present.  */
 
 	    for (i = 1; i <= min; i++)
 	      {
@@ -1108,10 +1119,10 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
 
 	case OP_CHARS:
 	  {
-	    register int length = ecode[1];
+	    int length = ecode[1];
 	    ecode += 2;
 
-#ifdef DEBUG			/* Sigh. Some compilers never learn. */
+#ifdef DEBUG
 	    if (eptr >= md->end_subject)
 	      printf ("matching subject <null> against pattern ");
 	    else
@@ -1676,7 +1687,8 @@ match (eptr, ecode, offset_top, md, ims, eptrb, flags)
          loop. */
 
     }				/* End of main loop */
-/* Control never reaches here */
+
+  /* Control never reaches here */
 }
 
 
@@ -1725,7 +1737,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
   match_data match_block;
   const uschar *start_bits = NULL;
   const uschar *bmtable = NULL;
-  const uschar *start_match = (const uschar *) subject + start_offset;
+  const uschar *start_match;
   const uschar *end_subject;
   const uschar *req_char_ptr = start_match - 1;
   BOOL using_temporary_offsets = FALSE;
@@ -1747,7 +1759,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
   match_block.start_pattern = re->code;
   match_block.start_subject = (const uschar *) subject;
   match_block.end_subject = match_block.start_subject + length;
-  end_subject = match_block.end_subject;
+  match_block.first_start = match_block.start_subject + start_offset;
 
   match_block.endonly = (re->options & PCRE_DOLLAR_ENDONLY) != 0;
 
@@ -1760,6 +1772,9 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
   match_block.lcc = re->tables + lcc_offset;
   match_block.ctypes = re->tables + ctypes_offset;
 
+  start_match = match_block.first_start;
+  end_subject = match_block.end_subject;
+
   /* For matches anchored to the end of the pattern, we can often avoid
      analyzing most of the pattern.  length > re->max_match_size is
      implied in the second condition, because start_offset > 0. */
@@ -1768,17 +1783,17 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
       && length - re->max_match_size > start_offset)
     start_match = (const uschar *) subject + length - re->max_match_size;
 
-/* The ims options can vary during the matching as a result of the presence
-   of (?ims) items in the pattern. They are kept in a local variable so that
-   restoring at the exit of a group is easy. */
+  /* The ims options can vary during the matching as a result of the presence
+     of (?ims) items in the pattern. They are kept in a local variable so that
+     restoring at the exit of a group is easy. */
 
   ims = re->options & (PCRE_CASELESS | PCRE_MULTILINE | PCRE_DOTALL);
 
-/* Set up the first character to match, if available. The first_char value is
-   never set for an anchored regular expression, but the anchoring may be forced
-   at run time, so we have to test for anchoring. The first char may be unset for
-   an unanchored pattern, of course. If there's no first char and the pattern was
-   studied, there may be a bitmap of possible first characters. */
+   /* Set up the first character to match, if available. The first_char value is
+      never set for an anchored regular expression, but the anchoring may be forced
+      at run time, so we have to test for anchoring. The first char may be unset for
+      an unanchored pattern, of course. If there's no first char and the pattern was
+      studied, there may be a bitmap of possible first characters. */
 
   if (!anchored)
     {
@@ -1801,10 +1816,10 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 	}
     }
 
-/* If the expression has got more back references than the offsets supplied can
-   hold, we get a temporary bit of working store to use during the matching.
-   Otherwise, we can use the vector supplied, rounding down its size to a multiple
-   of 3. */
+  /* If the expression has got more back references than the offsets supplied can
+     hold, we get a temporary bit of working store to use during the matching.
+     Otherwise, we can use the vector supplied, rounding down its size to a multiple
+     of 3. */
 
   ocount = offsetcount - (offsetcount % 3);
 
@@ -1824,34 +1839,31 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
   match_block.offset_max = (2 * ocount) / 3;
   match_block.offset_overflow = FALSE;
 
-/* Compute the minimum number of offsets that we need to reset each time. Doing
-   this makes a huge difference to execution time when there aren't many brackets
-   in the pattern. */
-
+  /* Compute the minimum number of offsets that we need to reset each time. Doing
+     this makes a huge difference to execution time when there aren't many brackets
+     in the pattern. */
   resetcount = 2 + re->top_bracket * 2;
   if (resetcount > offsetcount)
     resetcount = ocount;
 
-/* Reset the working variable associated with each extraction. These should
-   never be used unless previously set, but they get saved and restored, and so we
-   initialize them to avoid reading uninitialized locations. */
-
+  /* Reset the working variable associated with each extraction. These should
+     never be used unless previously set, but they get saved and restored, and so we
+     initialize them to avoid reading uninitialized locations. */
   if (match_block.offset_vector != NULL)
     {
-      register int *iptr = match_block.offset_vector + ocount;
-      register int *iend = iptr - resetcount / 2 + 1;
+      int *iptr = match_block.offset_vector + ocount;
+      int *iend = iptr - resetcount / 2 + 1;
       while (--iptr >= iend)
 	*iptr = -1;
     }
 
-/* For anchored or unanchored matches, there may be a "last known required
-   character" set. If the PCRE_CASELESS is set, implying that the match starts
-   caselessly, or if there are any changes of this flag within the regex, set up
-   both cases of the character. Otherwise set the two values the same, which will
-   avoid duplicate testing (which takes significant time). This covers the vast
-   majority of cases. It will be suboptimal when the case flag changes in a regex
-   and the required character in fact is caseful. */
-
+  /* For anchored or unanchored matches, there may be a "last known required
+     character" set. If the PCRE_CASELESS is set, implying that the match starts
+     caselessly, or if there are any changes of this flag within the regex, set up
+     both cases of the character. Otherwise set the two values the same, which will
+     avoid duplicate testing (which takes significant time). This covers the vast
+     majority of cases. It will be suboptimal when the case flag changes in a regex
+     and the required character in fact is caseful. */
   if ((re->options & PCRE_REQCHSET) != 0)
     {
       req_char = re->req_char;
@@ -1859,14 +1871,13 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 	(re->tables + fcc_offset)[req_char] : req_char;
     }
 
-/* Loop for handling unanchored repeated matching attempts; for anchored regexs
-   the loop runs just once. */
-
+  /* Loop for handling unanchored repeated matching attempts; for anchored regexs
+     the loop runs just once. */
   do
     {
       int rc;
-      register int *iptr = match_block.offset_vector;
-      register int *iend = iptr + resetcount;
+      int *iptr = match_block.offset_vector;
+      int *iend = iptr + resetcount;
 #ifdef DEBUG
       int skipped_chars = 0;
 #endif
@@ -1941,7 +1952,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 	{
 	  while (start_match < end_subject)
 	    {
-	      register int c = *start_match;
+	      int c = *start_match;
 	      if ((start_bits[c / 8] & (1 << (c & 7))) == 0)
 #ifdef DEBUG
 		skipped_chars++,
@@ -1952,7 +1963,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 	    }
 	}
 
-#ifdef DEBUG			/* Sigh. Some compilers never learn. */
+#ifdef DEBUG
       if (skipped_chars)
 	printf (">>>>> Skipped %d chars to reach first character\n",
 		skipped_chars);
@@ -1974,7 +1985,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 
       if (req_char >= 0)
 	{
-	  register const uschar *p = start_match + ((first_char >= 0) ? 1 : 0);
+	  const uschar *p = start_match + ((first_char >= 0) ? 1 : 0);
 
 	  /* We don't need to repeat the search if we haven't yet reached the
 	     place we found it at last time. */
@@ -2001,7 +2012,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
 		{
 		  while (p < end_subject)
 		    {
-		      register int pp = *p++;
+		      int pp = *p++;
 		      if (pp == req_char || pp == req_char2)
 			{
 			  p--;
@@ -2065,8 +2076,7 @@ pcre_exec (re, extra, subject, length, start_offset, options, offsets, offsetcou
       return rc;
     }
 
-/* This "while" is the end of the "do" above */
-
+  /* This "while" is the end of the "do" above */
   while (!anchored &&
 	 match_block.errorcode == PCRE_ERROR_NOMATCH &&
 	 start_match++ < end_subject);
