@@ -83,7 +83,7 @@ static const short int escapes[] =
 {
   0, 0, 0, 0, 0, 0, 0, 0,	/* 0 - 7 */
   0, 0, ':', ';', '<', '=', '>', '?',	/* 8 - ? */
-  '@', -ESC_A, -ESC_B, 0, -ESC_D, 0, 0, 0,	/* @ - G */
+  '@', -ESC_A, -ESC_B, 0, -ESC_D, 0, 0, -ESC_G,	/* @ - G */
   0, 0, 0, 0, 0, 0, 0, 0,	/* H - O */
   0, 0, 0, -ESC_S, 0, 0, 0, -ESC_W,	/* P - W */
   0, 0, -ESC_Z, '[', '\\', ']', '^', '_',	/* X - _ */
@@ -93,47 +93,6 @@ static const short int escapes[] =
   0, 0, -ESC_z			/* x - z */
 };
 
-/* Tables of names of POSIX character classes and their lengths. The list is
-   terminated by a zero length entry. The first three must be alpha, upper, lower,
-   as this is assumed for handling case independence. */
-
-static const char *posix_names[] =
-{
-  "alpha", "lower", "upper",
-  "alnum", "ascii", "cntrl", "digit", "graph",
-  "print", "punct", "space", "word", "xdigit"};
-
-static const uschar posix_name_lengths[] =
-{
-  5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 4, 6, 0};
-
-/* Table of class bit maps for each POSIX class; up to three may be combined
-   to form the class. */
-
-static const int posix_class_maps[] =
-{
-  cbit_lower, cbit_upper, -1,	/* alpha */
-  cbit_lower, -1, -1,		/* lower */
-  cbit_upper, -1, -1,		/* upper */
-  cbit_digit, cbit_lower, cbit_upper,	/* alnum */
-  cbit_print, cbit_cntrl, -1,	/* ascii */
-  cbit_cntrl, -1, -1,		/* cntrl */
-  cbit_digit, -1, -1,		/* digit */
-  cbit_graph, -1, -1,		/* graph */
-  cbit_print, -1, -1,		/* print */
-  cbit_punct, -1, -1,		/* punct */
-  cbit_space, -1, -1,		/* space */
-  cbit_word, -1, -1,		/* word */
-  cbit_xdigit, -1, -1		/* xdigit */
-};
-
-
-/* Definition to allow mutual recursion */
-
-static BOOL
-  compile_regex (int, int, int *, uschar **, const uschar **, const char **,
-		 BOOL, int, int *, int *, compile_data *);
-
 
 
 /*************************************************
@@ -148,6 +107,7 @@ static BOOL
 void *(*pcre_malloc) (size_t) = malloc;
 void (*pcre_free) (void *) = free;
 
+#include "regcomp.c"
 
 /*************************************************
 *            Handle escapes                      *
@@ -186,20 +146,17 @@ check_escape (ptrptr, errorptr, bracount, options, isclass, cd)
 
   c = *(++ptr);
 
-/* Digits or letters may have special meaning; all others are literals. */
-
+  /* Digits or letters may have special meaning; all others are literals. */
   if (c < '0' || c > 'z')
     {
     }
 
-/* Do an initial lookup in a table. A non-zero result is something that can be
-   returned immediately. Otherwise further processing may be required. */
-
+  /* Do an initial lookup in a table. A non-zero result is something that can be
+     returned immediately. Otherwise further processing may be required. */
   else if ((i = escapes[c - '0']) != 0)
     c = i;
 
-/* Escapes that need further processing, or are illegal. */
-
+  /* Escapes that need further processing, or are illegal. */
   else
     {
       const uschar *oldptr;
@@ -316,371 +273,6 @@ check_escape (ptrptr, errorptr, bracount, options, isclass, cd)
 }
 
 
-
-/*************************************************
-*            Check for counted repeat            *
-*************************************************/
-
-/* This function is called when a '{' is encountered in a place where it might
-   start a quantifier. It looks ahead to see if it really is a quantifier or not.
-   It is only a quantifier if it is one of the forms {ddd} {ddd,} or {ddd,ddd}
-   where the ddds are digits.
-
-   Arguments:
-   p         pointer to the first char after '{'
-   cd        pointer to char tables block
-
-   Returns:    TRUE or FALSE
- */
-
-static BOOL
-is_counted_repeat (p, cd)
-     const uschar *p;
-     compile_data *cd;
-{
-  if (p > cd->end)
-    return FALSE;
-  if ((cd->ctypes[*p++] & ctype_digit) == 0)
-    return FALSE;
-  while (p <= cd->end && (cd->ctypes[*p] & ctype_digit) != 0)
-    p++;
-
-  /* Check end of regexp */
-  if (p > cd->end)
-    return FALSE;
-  if (p == cd->end)
-    return (*p == '}');
-
-  /* Not at end of regexp */
-  if (*p == '}')
-    return TRUE;
-  if (*p++ != ',')
-    return FALSE;
-
-  /* Check again for end of regexp */
-  if (p == cd->end)
-    return (*p == '}');
-  if (*p == '}')
-    return TRUE;
-
-  if (p > cd->end)
-    return FALSE;
-  if ((cd->ctypes[*p++] & ctype_digit) == 0)
-    return FALSE;
-  while (p <= cd->end && (cd->ctypes[*p] & ctype_digit) != 0)
-    p++;
-
-  return (p <= cd->end && *p == '}');
-}
-
-
-
-/*************************************************
-*         Read repeat counts                     *
-*************************************************/
-
-/* Read an item of the form {n,m} and return the values. This is called only
-   after is_counted_repeat() has confirmed that a repeat-count quantifier exists,
-   so the syntax is guaranteed to be correct, but we need to check the values.
-
-   Arguments:
-   p          pointer to first char after '{'
-   minp       pointer to int for min
-   maxp       pointer to int for max
-   returned as -1 if no max
-   errorptr   points to pointer to error message
-   cd         pointer to character tables clock
-
-   Returns:     pointer to '}' on success;
-   current ptr on error, with errorptr set
- */
-
-static const uschar *
-read_repeat_counts (p, minp, maxp, errorptr, cd)
-     const uschar *p;
-     int *minp;
-     int *maxp;
-     const char **errorptr;
-     compile_data *cd;
-{
-  int min = 0;
-  int max = -1;
-
-  while ((cd->ctypes[*p] & ctype_digit) != 0)
-    min = min * 10 + *p++ - '0';
-
-  if (*p == '}')
-    max = min;
-  else
-    {
-      if (*(++p) != '}')
-	{
-	  max = 0;
-	  while ((cd->ctypes[*p] & ctype_digit) != 0)
-	    max = max * 10 + *p++ - '0';
-	  if (max < min)
-	    {
-	      *errorptr = pcre_estrings[4];
-	      return p;
-	    }
-	}
-    }
-
-/* Do paranoid checks, then fill in the required variables, and pass back the
-   pointer to the terminating '}'. */
-
-  if (min > 65535 || max > 65535)
-    *errorptr = pcre_estrings[5];
-  else
-    {
-      *minp = min;
-      *maxp = max;
-    }
-  return p;
-}
-
-
-
-/*************************************************
-*        Find the fixed length of a pattern      *
-*************************************************/
-
-/* Scan a pattern and compute the fixed length of subject that will match it,
-   if the length is fixed. This is needed for dealing with backward assertions.
-
-   Arguments:
-   code     points to the start of the pattern (the bracket)
-   options  the compiling options
-
-   Returns:   the fixed length, or -1 if there is no fixed length
- */
-
-static int
-find_fixedlength (code, options)
-     uschar *code;
-     int options;
-{
-  int length = -1;
-
-  register int branchlength = 0;
-  register uschar *cc = code + 3;
-
-/* Scan along the opcodes for this branch. If we get to the end of the
-   branch, check the length against that of the other branches. */
-
-  for (;;)
-    {
-      int d;
-      register int op = *cc;
-      if (op >= OP_BRA)
-	op = OP_BRA;
-
-      switch (op)
-	{
-	case OP_BRA:
-	case OP_ONCE:
-	case OP_COND:
-	  d = find_fixedlength (cc, options);
-	  if (d < 0)
-	    return -1;
-	  branchlength += d;
-	  do
-	    cc += (cc[1] << 8) + cc[2];
-	  while (*cc == OP_ALT);
-	  cc += 3;
-	  break;
-
-	  /* Reached end of a branch; if it's a ket it is the end of a nested
-	     call. If it's ALT it is an alternation in a nested call. If it is
-	     END it's the end of the outer call. All can be handled by the same code. */
-
-	case OP_ALT:
-	case OP_KET:
-	case OP_KET_MAXSTAR:
-	case OP_KET_MINSTAR:
-	case OP_END:
-	  if (length < 0)
-	    length = branchlength;
-	  else if (length != branchlength)
-	    return -1;
-	  if (*cc != OP_ALT)
-	    return length;
-	  cc += 3;
-	  branchlength = 0;
-	  break;
-
-	  /* Skip over assertive subpatterns */
-
-	case OP_ASSERT:
-	case OP_ASSERT_NOT:
-	case OP_ASSERTBACK:
-	case OP_ASSERTBACK_NOT:
-	  do
-	    cc += (cc[1] << 8) + cc[2];
-	  while (*cc == OP_ALT);
-	  cc += 3;
-	  break;
-
-	  /* Skip over things that don't match chars */
-
-	case OP_REVERSE:
-	case OP_BRANUMBER:
-	case OP_CREF:
-	  cc++;
-	  /* Fall through */
-
-	case OP_OPT:
-	  cc++;
-	  /* Fall through */
-
-	case OP_SOD:
-	case OP_EOD:
-	case OP_EODN:
-	case OP_CIRC:
-	case OP_DOLL:
-	case OP_NOT_WORD_BOUNDARY:
-	case OP_WORD_BOUNDARY:
-	  cc++;
-	  break;
-
-	  /* Handle char strings. */
-
-	case OP_CHARS:
-	  branchlength += *(++cc);
-	  cc += *cc + 1;
-	  break;
-
-	  /* Handle exact repetitions */
-
-	case OP_EXACT:
-	case OP_TYPEEXACT:
-	  branchlength += (cc[1] << 8) + cc[2];
-	  cc += 4;
-	  break;
-
-	  /* Handle single-char matchers */
-
-	case OP_TYPE:
-	case OP_TYPENOT:
-	  branchlength++;
-	  cc += 2;
-	  break;
-
-	case OP_ANY:
-	  branchlength++;
-	  cc++;
-	  break;
-
-	  /* Check a class for variable quantification */
-
-	case OP_CLASS:
-	  branchlength++;
-	  cc += 33;
-	  break;
-
-	case OP_CL_MAXSTAR:
-	case OP_CL_MINSTAR:
-	case OP_CL_MAXQUERY:
-	case OP_CL_MINQUERY:
-	  return -1;
-
-	case OP_CL_MAXRANGE:
-	case OP_CL_MINRANGE:
-	  cc += 33;
-	  if ((cc[1] << 8) + cc[2] != (cc[3] << 8) + cc[4])
-	    return -1;
-
-	  branchlength += (cc[1] << 8) + cc[2];
-	  cc += 4;
-	  break;
-
-	  /* Anything else is variable length */
-
-	default:
-	  return -1;
-	}
-    }
-/* Control never gets here */
-}
-
-
-
-
-/*************************************************
-*           Check for POSIX class syntax         *
-*************************************************/
-
-/* This function is called when the sequence "[:" or "[." or "[=" is
-   encountered in a character class. It checks whether this is followed by an
-   optional ^ and then a sequence of letters, terminated by a matching ":]" or
-   ".]" or "=]".
-
-   Argument:
-   ptr      pointer to the initial [
-   endptr   where to return the end pointer
-   cd       pointer to compile data
-
-   Returns:   TRUE or FALSE
- */
-
-static BOOL
-check_posix_syntax (ptr, endptr, cd)
-     const uschar *ptr;
-     const uschar **endptr;
-     compile_data *cd;
-{
-  int terminator;		/* Don't combine these lines; the Solaris cc */
-  terminator = *(++ptr);	/* compiler warns about "non-constant" initializer. */
-  if (*(++ptr) == '^')
-    ptr++;
-
-  /* Use < because we have to look one character ahead */
-  while (ptr < cd->end && (cd->ctypes[*ptr] & ctype_letter) != 0)
-    ptr++;
-  if (ptr < cd->end && *ptr == terminator && ptr[1] == ']')
-    {
-      *endptr = ptr;
-      return TRUE;
-    }
-  return FALSE;
-}
-
-
-
-
-/*************************************************
-*          Check POSIX class name                *
-*************************************************/
-
-/* This function is called to check the name given in a POSIX-style class entry
-   such as [:alnum:].
-
-   Arguments:
-   ptr        points to the first letter
-   len        the length of the name
-
-   Returns:     a value representing the name, or -1 if unknown
- */
-
-static int
-check_posix_name (ptr, len)
-     const uschar *ptr;
-     int len;
-{
-  register int yield = 0;
-  while (posix_name_lengths[yield] != 0)
-    {
-      if (len == posix_name_lengths[yield] &&
-	  strncmp ((const char *) ptr, posix_names[yield], len) == 0)
-	return yield;
-      yield++;
-    }
-  return -1;
-}
-
-
-
-
 /*************************************************
 *           Compile one branch                   *
 *************************************************/
@@ -703,7 +295,7 @@ check_posix_name (ptr, len)
  */
 
 static BOOL
-compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqchar, countlits, cd)
+compile_branch_perl (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqchar, countlits, cd)
      int options;
      int *brackets;
      uschar **codeptr;
@@ -721,26 +313,23 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
   int prevreqchar;
   int condcount = 0;
   int subcountlits = 0;
-  register int c;
-  register uschar *code = *codeptr;
-  uschar *tempcode;
+  int c;
+  uschar *start;
+  uschar *code = *codeptr;
   const uschar *ptr = *ptrptr;
   const uschar *tempptr;
   uschar *previous = NULL;
   uschar class[32];
 
-/* Set up the default and non-default settings for greediness */
-
+  /* Set up the default and non-default settings for greediness */
   greedy_default = ((options & PCRE_UNGREEDY) != 0);
   greedy_non_default = greedy_default ^ 1;
 
-/* Initialize no required char, and count of literals */
-
+  /* Initialize no required char, and count of literals */
   *reqchar = prevreqchar = -1;
   *countlits = 0;
 
-/* Switch on next character until the end of the branch */
-
+  /* Switch on next character until the end of the branch */
   for (; ptr <= cd->end; ptr++)
     {
       BOOL negate_class;
@@ -849,7 +438,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 		{
 		  BOOL local_negate = FALSE;
 		  int posix_class, i;
-		  register const uschar *cbits = cd->cbits;
+		  const uschar *cbits = cd->cbits;
 
 		  if (ptr[1] != ':')
 		    {
@@ -915,7 +504,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 		    c = '\b';
 		  else if (c < 0)
 		    {
-		      register const uschar *cbits = cd->cbits;
+		      const uschar *cbits = cd->cbits;
 		      class_charcount = 10;
 		      switch (-c)
 			{
@@ -1073,6 +662,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 	case '{':
 	  if (!is_counted_repeat (ptr + 1, cd))
 	    goto NORMAL_CHAR;
+	  repeat_min = repeat_max = 1;
 	  ptr = read_repeat_counts (ptr + 1, &repeat_min, &repeat_max, errorptr, cd);
 	  if (*errorptr != NULL)
 	    goto FAILED;
@@ -1301,7 +891,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 	  else if ((int) *previous >= OP_BRA || (int) *previous == OP_ONCE ||
 		   (int) *previous == OP_COND)
 	    {
-	      register int i;
+	      int i;
 	      int ketoffset = 0;
 	      int len = code - previous;
 	      uschar *bralink = NULL;
@@ -1314,7 +904,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 
 	      if (repeat_max == -1)
 		{
-		  register uschar *ket = previous;
+		  uschar *ket = previous;
 		  do
 		    ket += (ket[1] << 8) + ket[2];
 		  while (*ket != OP_KET);
@@ -1667,21 +1257,19 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 		bravalue = OP_BRA + *brackets;
 	    }
 
-	  /* Process nested bracketed re. Assertions may not be repeated, but other
-	     kinds can be. We copy code into a non-register variable in order to be able
-	     to pass its address because some compilers complain otherwise. Pass in a
-	     new setting for the ims options if they have changed. */
+	  /* Process nested bracketed re. Assertions may not be repeated, but other kinds
+	     can be. Pass in a new setting for the ims options if they have changed. */
 
 	  previous = (bravalue >= OP_ONCE) ? code : NULL;
 	  *code = bravalue;
-	  tempcode = code;
+	  start = code;
 
 	  if (!compile_regex (
 			       options | PCRE_INGROUP,	/* Set for all nested groups */
 			 ((options & PCRE_IMS) != (newoptions & PCRE_IMS)) ?
 			       newoptions & PCRE_IMS : -1,	/* Pass ims options if changed */
 			       brackets,	/* Capturing bracket count */
-			       &tempcode,	/* Where to put code (updated) */
+			       &code,	/* Where to put code (updated) */
 			       &ptr,	/* Input pointer (updated) */
 			       errorptr,	/* Where to put an error message */
 			       (bravalue == OP_ASSERTBACK ||
@@ -1689,11 +1277,12 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 			       skipbytes,		/* Skip over OP_COND/OP_BRANUMBER */
 			       &subreqchar,	/* For possible last char */
 			       &subcountlits,	/* For literal count */
-			       cd))	/* Tables block */
+			       cd,		/* Tables block */
+			       compile_branch_perl))
 	    goto FAILED;
 
 	  /* At the end of compiling, code is still pointing to the start of the
-	     group, while tempcode has been updated to point past the end of the group
+	     group, while code has been updated to point past the end of the group
 	     and any option resetting that may follow it. The pattern pointer (ptr)
 	     is on the bracket. */
 
@@ -1702,7 +1291,7 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 
 	  else if (bravalue == OP_COND)
 	    {
-	      uschar *tc = code;
+	      uschar *tc = start;
 	      condcount = 0;
 
 	      do
@@ -1737,10 +1326,6 @@ compile_branch (options, brackets, codeptr, ptrptr, errorptr, optchanged, reqcha
 		*countlits += subcountlits;
 	    }
 
-	  /* Now update the main code pointer to the end of the group. */
-
-	  code = tempcode;
-
 	  /* Error if hit end of pattern */
 
 	  if (ptr > cd->end || *ptr != ')')
@@ -1870,10 +1455,9 @@ SUCCEEDED:
   *ptrptr = ptr;
   return TRUE;
 
-/* Control never reaches here by falling through, only by a goto for all the
-   error states. Pass back the position in the pattern so that it can be displayed
-   to the user for diagnosing the error. */
-
+  /* Control never reaches here by falling through, only by a goto for all the
+     error states. Pass back the position in the pattern so that it can be displayed
+     to the user for diagnosing the error. */
 FAILED:
   *ptrptr = ptr;
   return FALSE;
@@ -1881,417 +1465,6 @@ FAILED:
 
 
 
-
-/*************************************************
-*     Compile sequence of alternatives           *
-*************************************************/
-
-/* On entry, ptr is pointing past the bracket character, but on return
-   it points to the closing bracket, or vertical bar, or end of string.
-   The code variable is pointing at the byte into which the BRA operator has been
-   stored. If the ims options are changed at the start (for a (?ims: group) or
-   during any branch, we need to insert an OP_OPT item at the start of every
-   following branch to ensure they get set correctly at run time, and also pass
-   the new options into every subsequent branch compile.
-
-   Argument:
-   options     the option bits
-   optchanged  new ims options to set as if (?ims) were at the start, or -1
-   for no change
-   brackets    -> int containing the number of capturing brackets used
-   codeptr     -> the address of the current code pointer
-   ptrptr      -> the address of the current pattern pointer
-   errorptr    -> pointer to error message
-   lookbehind  TRUE if this is a lookbehind assertion
-   skipbytes   skip this many bytes at start (for OP_COND, OP_BRANUMBER)
-   reqchar     -> place to put the last required character, or a negative number
-   countlits   -> place to put the shortest literal count of any branch
-   cd          points to the data block with tables pointers
-
-   Returns:      TRUE on success
- */
-
-static BOOL
-compile_regex (options, optchanged, brackets, codeptr, ptrptr, errorptr, lookbehind, skipbytes, reqchar, countlits, cd)
-     int options;
-     int optchanged;
-     int *brackets;
-     uschar **codeptr;
-     const uschar **ptrptr;
-     const char **errorptr;
-     BOOL lookbehind;
-     int skipbytes;
-     int *reqchar;
-     int *countlits;
-     compile_data *cd;
-{
-  const uschar *ptr = *ptrptr;
-  uschar *code = *codeptr;
-  uschar *last_branch = code;
-  uschar *start_bracket = code;
-  uschar *reverse_count = NULL;
-  int oldoptions = options & PCRE_IMS;
-  int branchreqchar, branchcountlits;
-
-  *reqchar = -1;
-  *countlits = INT_MAX;
-  code += 3 + skipbytes;
-
-/* Loop for each alternative branch */
-
-  for (;;)
-    {
-      int length;
-
-      /* Handle change of options */
-
-      if (optchanged >= 0)
-	{
-	  *code++ = OP_OPT;
-	  *code++ = optchanged;
-	  options = (options & ~PCRE_IMS) | optchanged;
-	}
-
-      /* Set up dummy OP_REVERSE if lookbehind assertion */
-
-      if (lookbehind)
-	{
-	  *code++ = OP_REVERSE;
-	  reverse_count = code;
-	  *code++ = 0;
-	  *code++ = 0;
-	}
-
-      /* Now compile the branch */
-
-      if (!compile_branch (options, brackets, &code, &ptr, errorptr, &optchanged,
-			   &branchreqchar, &branchcountlits, cd))
-	{
-	  *ptrptr = ptr;
-	  return FALSE;
-	}
-
-      /* Fill in the length of the last branch */
-
-      length = code - last_branch;
-      last_branch[1] = length >> 8;
-      last_branch[2] = length & 255;
-
-      /* Save the last required character if all branches have the same; a current
-         value of -1 means unset, while -2 means "previous branch had no last required
-         char".  */
-
-      if (*reqchar != -2)
-	{
-	  if (branchreqchar >= 0)
-	    {
-	      if (*reqchar == -1)
-		*reqchar = branchreqchar;
-	      else if (*reqchar != branchreqchar)
-		*reqchar = -2;
-	    }
-	  else
-	    *reqchar = -2;
-	}
-
-      /* Keep the shortest literal count */
-
-      if (branchcountlits < *countlits)
-	*countlits = branchcountlits;
-      DPRINTF (("literal count = %d min=%d\n", branchcountlits, *countlits));
-
-      /* If lookbehind, check that this branch matches a fixed-length string,
-         and put the length into the OP_REVERSE item. Temporarily mark the end of
-         the branch with OP_END. */
-
-      if (lookbehind)
-	{
-	  *code = OP_END;
-	  length = find_fixedlength (last_branch, options);
-	  DPRINTF (("fixed length = %d\n", length));
-	  if (length < 0)
-	    {
-	      *errorptr = pcre_estrings[25];
-	      *ptrptr = ptr;
-	      return FALSE;
-	    }
-	  reverse_count[0] = (length >> 8);
-	  reverse_count[1] = length & 255;
-	}
-
-      /* Reached end of expression, either ')' or end of pattern. Insert a
-         terminating ket and the length of the whole bracketed item, and return,
-         leaving the pointer at the terminating char. If any of the ims options
-         were changed inside the group, compile a resetting op-code following. */
-
-      if (ptr > cd->end || *ptr != '|')
-	{
-	  length = code - start_bracket;
-	  *code++ = OP_KET;
-	  *code++ = length >> 8;
-	  *code++ = length & 255;
-	  if (optchanged >= 0)
-	    {
-	      *code++ = OP_OPT;
-	      *code++ = oldoptions;
-	    }
-	  *codeptr = code;
-	  *ptrptr = ptr;
-	  return TRUE;
-	}
-
-      /* Another branch follows; insert an "or" node and advance the pointer. */
-
-      *code = OP_ALT;
-      last_branch = code;
-      code += 3;
-      ptr++;
-    }
-/* Control never reaches here */
-}
-
-
-
-
-/*************************************************
-*      Find first significant op code            *
-*************************************************/
-
-/* This is called by several functions that scan a compiled expression looking
-   for a fixed first character, or an anchoring op code etc. It skips over things
-   that do not influence this. For one application, a change of caseless option is
-   important.
-
-   Arguments:
-   code       pointer to the start of the group
-   options    pointer to external options
-   optbit     the option bit whose changing is significant, or
-   zero if none are
-   optstop    TRUE to return on option change, otherwise change the options
-   value and continue
-
-   Returns:     pointer to the first significant opcode
- */
-
-static const uschar *
-first_significant_code (code, options, optbit, optstop)
-     const uschar *code;
-     int *options;
-     int optbit;
-     BOOL optstop;
-{
-  for (;;)
-    {
-      switch ((int) *code)
-	{
-	case OP_OPT:
-	  if (optbit > 0 && ((int) code[1] & optbit) != (*options & optbit))
-	    {
-	      if (optstop)
-		return code;
-	      *options = (int) code[1];
-	    }
-	  code += 2;
-	  break;
-
-	case OP_CREF:
-	case OP_BRANUMBER:
-	  code += 3;
-	  break;
-
-	case OP_WORD_BOUNDARY:
-	case OP_NOT_WORD_BOUNDARY:
-	  code++;
-	  break;
-
-	case OP_ASSERT_NOT:
-	case OP_ASSERTBACK:
-	case OP_ASSERTBACK_NOT:
-	  do
-	    code += (code[1] << 8) + code[2];
-	  while (*code == OP_ALT);
-	  code += 3;
-	  break;
-
-	default:
-	  return code;
-	}
-    }
-/* Control never reaches here */
-}
-
-
-
-
-/*************************************************
-*          Check for anchored expression         *
-*************************************************/
-
-/* Try to find out if this is an anchored regular expression. Consider each
-   alternative branch. If they all start with OP_SOD or OP_CIRC, or with a bracket
-   all of whose alternatives start with OP_SOD or OP_CIRC (recurse ad lib), then
-   it's anchored. However, if this is a multiline pattern, then only OP_SOD
-   counts, since OP_CIRC can match in the middle.
-
-   A branch is also implicitly anchored if it starts with .* and DOTALL is set,
-   because that will try the rest of the pattern at all possible matching points,
-   so there is no point trying them again.
-
-   Arguments:
-   code       points to start of expression (the bracket)
-   options    points to the options setting
-
-   Returns:     TRUE or FALSE
- */
-
-static BOOL
-is_anchored (code, options)
-     register const uschar *code;
-     int *options;
-{
-  do
-    {
-      const uschar *scode = first_significant_code (code + 3, options,
-						    PCRE_MULTILINE, FALSE);
-      register int op = *scode;
-      if (op >= OP_BRA || op == OP_ASSERT || op == OP_ONCE || op == OP_COND)
-	{
-	  if (!is_anchored (scode, options))
-	    return FALSE;
-	}
-      else if ((op == OP_TYPE_MAXSTAR || op == OP_TYPE_MINSTAR) &&
-	       (*options & PCRE_DOTALL) != 0)
-	{
-	  if (scode[1] != OP_ANY)
-	    return FALSE;
-	}
-      else if (op != OP_SOD &&
-	       ((*options & PCRE_MULTILINE) != 0 || op != OP_CIRC))
-	return FALSE;
-      code += (code[1] << 8) + code[2];
-    }
-  while (*code == OP_ALT);
-  return TRUE;
-}
-
-
-
-/*************************************************
-*         Check for starting with ^ or .*        *
-*************************************************/
-
-/* This is called to find out if every branch starts with ^ or .* so that
-   "first char" processing can be done to speed things up in multiline
-   matching and for non-DOTALL patterns that start with .* (which must start at
-   the beginning or after \n).
-
-   Argument:  points to start of expression (the bracket)
-   Returns:   TRUE or FALSE
- */
-
-static BOOL
-is_startline (code)
-     const uschar *code;
-{
-  do
-    {
-      const uschar *scode = first_significant_code (code + 3, NULL, 0, FALSE);
-      register int op = *scode;
-      if (op >= OP_BRA || op == OP_ASSERT || op == OP_ONCE || op == OP_COND)
-	{
-	  if (!is_startline (scode))
-	    return FALSE;
-	}
-      else if (op == OP_TYPE_MAXSTAR || op == OP_TYPE_MINSTAR)
-	{
-	  if (scode[1] != OP_ANY)
-	    return FALSE;
-	}
-      else if (op != OP_CIRC)
-	return FALSE;
-      code += (code[1] << 8) + code[2];
-    }
-  while (*code == OP_ALT);
-  return TRUE;
-}
-
-
-
-/*************************************************
-*          Check for fixed first char            *
-*************************************************/
-
-/* Try to find out if there is a fixed first character. This is called for
-   unanchored expressions, as it speeds up their processing quite considerably.
-   Consider each alternative branch. If they all start with the same char, or with
-   a bracket all of whose alternatives start with the same char (recurse ad lib),
-   then we return that char, otherwise -1.
-
-   Arguments:
-   code       points to start of expression (the bracket)
-   options    pointer to the options (used to check casing changes)
-
-   Returns:     -1 or the fixed first char
- */
-
-static int
-find_firstchar (code, options)
-     const uschar *code;
-     int *options;
-{
-  register int c = -1;
-  do
-    {
-      int d;
-      const uschar *scode = first_significant_code (code + 3, options,
-						    PCRE_CASELESS, TRUE);
-      register int op = *scode;
-
-      if (op >= OP_BRA)
-	op = OP_BRA;
-
-      switch (op)
-	{
-	default:
-	  return -1;
-
-	case OP_BRA:
-	case OP_ASSERT:
-	case OP_ONCE:
-	case OP_COND:
-	  if ((d = find_firstchar (scode, options)) < 0)
-	    return -1;
-	  if (c < 0)
-	    c = d;
-	  else if (c != d)
-	    return -1;
-	  break;
-
-	case OP_EXACT:		/* Fall through */
-	  scode++;
-
-	case OP_CHARS:		/* Fall through */
-	  scode++;
-
-	case OP_MAXPLUS:
-	case OP_MINPLUS:
-	  if (c < 0)
-	    c = scode[1];
-	  else if (c != scode[1])
-	    return -1;
-	  break;
-	}
-
-      code += (code[1] << 8) + code[2];
-    }
-  while (*code == OP_ALT);
-  return c;
-}
-
-
-
-
-
 /*************************************************
 *        Compile a Regular Expression            *
 *************************************************/
@@ -2353,15 +1526,13 @@ pcre_compile_nuls (pattern, patlen, options,
   uschar *code_base, *code_end;
 #endif
 
-/* We can't pass back an error message if errorptr is NULL; I guess the best we
-   can do is just return NULL. */
-
+  /* We can't pass back an error message if errorptr is NULL; I guess the best we
+     can do is just return NULL. */
   if (errorptr == NULL)
     return NULL;
   *errorptr = NULL;
 
-/* However, we can give a message for this error */
-
+  /* However, we can give a message for this error */
   if (erroroffset == NULL)
     {
       *errorptr = pcre_estrings[16];
@@ -2375,8 +1546,7 @@ pcre_compile_nuls (pattern, patlen, options,
       return NULL;
     }
 
-/* Set up pointers to the individual character tables */
-
+  /* Set up pointers to the individual character tables */
   if (!tables)
     {
       if (!def_tables)
@@ -2389,19 +1559,17 @@ pcre_compile_nuls (pattern, patlen, options,
   compile_block.ctypes = tables + ctypes_offset;
   compile_block.end = pattern + patlen - 1;
 
-/* Reflect pattern for debugging output */
-
+  /* Reflect pattern for debugging output */
   DPRINTF (("------------------------------------------------------------------\n"));
   DFWRITE ((pattern, 1, patlen, stdout));
   DPRINTF (("\n"));
 
-/* The first thing to do is to make a pass over the pattern to compute the
-   amount of store required to hold the compiled code. This does not have to be
-   perfect as long as errors are overestimates. At the same time we can detect any
-   internal flag settings. Make an attempt to correct for any counted white space
-   if an "extended" flag setting appears late in the pattern. We can't be so
-   clever for #-comments. */
-
+  /* The first thing to do is to make a pass over the pattern to compute the
+     amount of store required to hold the compiled code. This does not have to be
+     perfect as long as errors are overestimates. At the same time we can detect any
+     internal flag settings. Make an attempt to correct for any counted white space
+     if an "extended" flag setting appears late in the pattern. We can't be so
+     clever for #-comments. */
   for (ptr = (const uschar *) pattern; ptr <= compile_block.end; ptr++)
     {
       int min, max;
@@ -2461,6 +1629,7 @@ pcre_compile_nuls (pattern, patlen, options,
 	      if (ptr < compile_block.end && ptr[1] == '{' &&
 		  is_counted_repeat (ptr + 2, &compile_block))
 		{
+		  min = max = 1;
 		  ptr = read_repeat_counts (ptr + 2, &min, &max, errorptr, &compile_block);
 		  if (*errorptr != NULL)
 		    goto PCRE_ERROR_RETURN;
@@ -2497,6 +1666,7 @@ pcre_compile_nuls (pattern, patlen, options,
 	case '{':
 	  if (!is_counted_repeat (ptr + 1, &compile_block))
 	    goto NORMAL_CHAR;
+	  min = max = 1;
 	  ptr = read_repeat_counts (ptr + 1, &min, &max, errorptr, &compile_block);
 	  if (*errorptr != NULL)
 	    goto PCRE_ERROR_RETURN;
@@ -2575,6 +1745,7 @@ pcre_compile_nuls (pattern, patlen, options,
 
 	      if (ptr < compile_block.end && ptr[1] == '{' && is_counted_repeat (ptr + 2, &compile_block))
 		{
+		  min = max = 1;
 		  ptr = read_repeat_counts (ptr + 2, &min, &max, errorptr, &compile_block);
 		  if (*errorptr != NULL)
 		    goto PCRE_ERROR_RETURN;
@@ -2850,6 +2021,7 @@ pcre_compile_nuls (pattern, patlen, options,
 	    else if ((c = ptr[1]) == '{' &&
 		     is_counted_repeat (ptr + 2, &compile_block))
 	      {
+		minval = maxval = 1;
 		ptr = read_repeat_counts (ptr + 2, &minval, &maxval, errorptr,
 					  &compile_block);
 		if (*errorptr != NULL)
@@ -2966,12 +2138,11 @@ pcre_compile_nuls (pattern, patlen, options,
       return NULL;
     }
 
-/* Compute the size of data block needed and get it, either from malloc or
-   externally provided function. We specify "code[0]" in the offsetof() expression
-   rather than just "code", because it has been reported that one broken compiler
-   fails on "code" because it is also an independent variable. It should make no
-   difference to the value of the offsetof(). */
-
+  /* Compute the size of data block needed and get it, either from malloc or
+     externally provided function. We specify "code[0]" in the offsetof() expression
+     rather than just "code", because it has been reported that one broken compiler
+     fails on "code" because it is also an independent variable. It should make no
+     difference to the value of the offsetof(). */
   size = length + offsetof (pcre, code[0]);
   re = (pcre *) (pcre_malloc) (size);
 
@@ -2981,34 +2152,30 @@ pcre_compile_nuls (pattern, patlen, options,
       return NULL;
     }
 
-/* Put in the magic number, and save the size, options, and table pointer */
-
+  /* Put in the magic number, and save the size, options, and table pointer */
   re->magic_number = MAGIC_NUMBER;
   re->size = size;
   re->options = options;
   re->tables = tables;
 
-/* Set up a starting, non-capturing bracket, then compile the expression. On
-   error, *errorptr will be set non-NULL, so we don't need to look at the result
-   of the function here. */
-
+  /* Set up a starting, non-capturing bracket, then compile the expression. On
+     error, *errorptr will be set non-NULL, so we don't need to look at the result
+     of the function here. */
   ptr = (const uschar *) pattern;
   code = re->code;
   *code = OP_BRA;
   bracount = 0;
   (void) compile_regex (options, -1, &bracount, &code, &ptr, errorptr, FALSE,
-			0, &reqchar, &countlits, &compile_block);
+			0, &reqchar, &countlits, &compile_block, compile_branch_perl);
   re->top_bracket = bracount;
   re->top_backref = top_backref;
 
-/* If not reached end of pattern on success, there's an excess bracket. */
-
+  /* If not reached end of pattern on success, there's an excess bracket. */
   if (*errorptr == NULL && ptr <= compile_block.end)
     *errorptr = pcre_estrings[22];
 
-/* Fill in the terminating state and check for disastrous overflow, but
-   if debugging, leave the test till after things are printed out. */
-
+  /* Fill in the terminating state and check for disastrous overflow, but
+     if debugging, leave the test till after things are printed out. */
   *code++ = OP_END;
 
 #ifndef DEBUG
@@ -3016,9 +2183,8 @@ pcre_compile_nuls (pattern, patlen, options,
       *errorptr = pcre_estrings[23];
 #endif
 
-/* Give an error if there's back reference to a non-existent capturing
-   subpattern. */
-
+  /* Give an error if there's back reference to a non-existent capturing
+     subpattern. */
   if (top_backref > re->top_bracket)
     *errorptr = pcre_estrings[15];
 
@@ -3032,16 +2198,15 @@ pcre_compile_nuls (pattern, patlen, options,
       return NULL;
     }
 
-/* If the anchored option was not passed, set flag if we can determine that the
-   pattern is anchored by virtue of ^ characters or \A or anything else (such as
-   starting with .* when DOTALL is set).
-
-   Otherwise, see if we can determine what the first character has to be, because
-   that speeds up unanchored matches no end. If not, see if we can set the
-   PCRE_STARTLINE flag. This is helpful for multiline matches when all branches
-   start with ^. and also when all branches start with .* for non-DOTALL matches.
- */
+  /* If the anchored option was not passed, set flag if we can determine that the
+     pattern is anchored by virtue of ^ characters or \A or anything else (such as
+     starting with .* when DOTALL is set).
 
+     Otherwise, see if we can determine what the first character has to be, because
+     that speeds up unanchored matches no end. If not, see if we can set the
+     PCRE_STARTLINE flag. This is helpful for multiline matches when all branches
+     start with ^. and also when all branches start with .* for non-DOTALL matches.
+   */
   if ((options & PCRE_ANCHORED) == 0)
     {
       int temp_options = options;
@@ -3060,9 +2225,8 @@ pcre_compile_nuls (pattern, patlen, options,
 	}
     }
 
-/* Save the last required character if there are at least two literal
-   characters on all paths, or if there is no first character setting. */
-
+  /* Save the last required character if there are at least two literal
+     characters on all paths, or if there is no first character setting. */
   if (reqchar >= 0 && (countlits > 1 || (re->options & PCRE_FIRSTSET) == 0))
     {
       re->req_char = reqchar;
@@ -3071,7 +2235,7 @@ pcre_compile_nuls (pattern, patlen, options,
 
   re->max_match_size = -1;
 
-/* Print out the compiled data for debugging */
+  /* Print out the compiled data for debugging */
 #ifdef DEBUG
   pcre_debug (re);
 
