@@ -39,62 +39,51 @@
 #include "internal.h"
 
 uschar pruned[] = {
-  OP_END,
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 
-  OP_BEG_WORD,	OP_END_WORD,    OP_SOD,    OP_NOT_WORD_BOUNDARY,
-  OP_WORD_BOUNDARY,	OP_EODN,	OP_EOD,    OP_OPT,
-  OP_CIRC,		OP_DOLL,	OP_ANY,
-  
-  OP_CHARS,
+  0,
   OP_ONCESTAR,    OP_ONCESTAR,    OP_ONCESTAR,
   OP_ONCEPLUS,    OP_ONCEPLUS,    OP_ONCEPLUS,
   OP_ONCEQUERY,   OP_ONCEQUERY,   OP_ONCEQUERY,
   OP_ONCEUPTO,    OP_ONCEUPTO,    OP_ONCEUPTO,
-  OP_EXACT,
+  0,
   
-  OP_NOT,
+  0,
   OP_NOT_ONCESTAR,    OP_NOT_ONCESTAR,    OP_NOT_ONCESTAR,
   OP_NOT_ONCEPLUS,    OP_NOT_ONCEPLUS,    OP_NOT_ONCEPLUS,
   OP_NOT_ONCEQUERY,   OP_NOT_ONCEQUERY,   OP_NOT_ONCEQUERY,
   OP_NOT_ONCEUPTO,    OP_NOT_ONCEUPTO,    OP_NOT_ONCEUPTO,
-  OP_NOTEXACT,
+  0,
   
-  OP_TYPE,
+  0,
   OP_TYPE_ONCESTAR,    OP_TYPE_ONCESTAR,    OP_TYPE_ONCESTAR,
   OP_TYPE_ONCEPLUS,    OP_TYPE_ONCEPLUS,    OP_TYPE_ONCEPLUS,
   OP_TYPE_ONCEQUERY,   OP_TYPE_ONCEQUERY,   OP_TYPE_ONCEQUERY,
   OP_TYPE_ONCEUPTO,    OP_TYPE_ONCEUPTO,    OP_TYPE_ONCEUPTO,
-  OP_TYPEEXACT,
+  0,
   
-  OP_TYPENOT,
+  0,
   OP_TYPENOT_ONCESTAR,    OP_TYPENOT_ONCESTAR,    OP_TYPENOT_ONCESTAR,
   OP_TYPENOT_ONCEPLUS,    OP_TYPENOT_ONCEPLUS,    OP_TYPENOT_ONCEPLUS,
   OP_TYPENOT_ONCEQUERY,   OP_TYPENOT_ONCEQUERY,   OP_TYPENOT_ONCEQUERY,
   OP_TYPENOT_ONCEUPTO,    OP_TYPENOT_ONCEUPTO,    OP_TYPENOT_ONCEUPTO,
-  OP_TYPENOTEXACT,
+  0,
   
-  OP_CLASS,
+  0,
   OP_CL_ONCESTAR,    OP_CL_ONCESTAR,    OP_CL_ONCESTAR,
   OP_CL_ONCEPLUS,    OP_CL_ONCEPLUS,    OP_CL_ONCEPLUS,
   OP_CL_ONCEQUERY,   OP_CL_ONCEQUERY,   OP_CL_ONCEQUERY,
   OP_CL_ONCERANGE,   OP_CL_ONCERANGE,   OP_CL_ONCERANGE,
   
-  OP_REF,
+  0,
   OP_REF_ONCESTAR,    OP_REF_ONCESTAR,    OP_REF_ONCESTAR,
   OP_REF_ONCEPLUS,    OP_REF_ONCEPLUS,    OP_REF_ONCEPLUS,
   OP_REF_ONCEQUERY,   OP_REF_ONCEQUERY,   OP_REF_ONCEQUERY,
   OP_REF_ONCERANGE,   OP_REF_ONCERANGE,   OP_REF_ONCERANGE,
   
-  OP_RECURSE,    OP_ALT,    OP_KET,
+  0, 0, 0,
   
-  OP_KET_ONCESTAR, OP_KET_ONCESTAR, OP_KET_ONCESTAR,
-  
-  OP_ASSERT,		OP_ASSERT_NOT,
-  OP_ASSERTBACK,	OP_ASSERTBACK_NOT,
-  OP_REVERSE,		OP_ONCE,
-  OP_COND,		OP_CREF,
-  OP_BRAZERO,		OP_BRAMINZERO,
-  OP_BRANUMBER
+  OP_KET_ONCESTAR, OP_KET_ONCESTAR, OP_KET_ONCESTAR
 };
 
 
@@ -191,29 +180,31 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 {
   const uschar *tcode = code + 3;
   BOOL try_next = TRUE;
-  int old_length;
 
   /* If the branch has more than one alternative, it's too
      complicated, so exit. */
   if (code[(code[1] << 8) + code[2]] == OP_ALT)
     return length;
 
-  while (try_next && length < 255)
+  do
     {
-      register int c = *tcode;
-      old_length = length;
+      int i;
 
-    NEXT_OPCODE:
-
-      switch (c)
+      switch (*tcode)
 	{
+	default:
+	  /* If a branch starts with a bracket, recurse to set the string
+	     from within them. That's all for this branch. */
+	  if ((int) *tcode >= OP_BRA)
+	    length = set_boyer_moore (tcode, bmtable, length, caseless, cd);
+	  return length;
+
 	  /* Skip over extended extraction bracket number */
 		
 	case OP_BRANUMBER:
 	  tcode += 3;
 	  break;
 
-
 	  /* Skip over lookbehind assertions */
 
 	case OP_ASSERTBACK:
@@ -261,13 +252,15 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 
 	case OP_EXACT:
 	  length += (tcode[1] << 8) + tcode[2];
+	  if (length > 255)
+	    length = 255;
 	  set_bmtable (bmtable, tcode[3], length, caseless, cd);
 	  tcode += 4;
 	  break;
 
 	case OP_CHARS:
 	  {
-	    register int nchars = tcode[1];
+	    int nchars = tcode[1];
 	    if (nchars > 255 - length)
 	      nchars = 255 - length;
 
@@ -283,9 +276,9 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 	  tcode++;
 	REPEATTYPENOT:
 	  length++;
-	  for (c = 0; c < 255; c++)
-	    if (!*tcode || !(cd->ctypes[c] & (1 << *tcode)))
-	      set_bmtable (bmtable, c, length, caseless, cd);
+	  for (i = 0; i < 255; i++)
+	    if (!*tcode || !(cd->ctypes[i] & (1 << *tcode)))
+	      set_bmtable (bmtable, i, length, caseless, cd);
 
 	  break;
 
@@ -293,9 +286,9 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 	  tcode++;
 	REPEATTYPE:
 	  length++;
-	  for (c = 0; c < 255; c++)
-	    if (cd->ctypes[c] & (*tcode ? 1 << *tcode : 0))
-	      set_bmtable (bmtable, c, length, caseless, cd);
+	  for (i = 0; i < 255; i++)
+	    if (cd->ctypes[i] & (*tcode ? 1 << *tcode : 0))
+	      set_bmtable (bmtable, i, length, caseless, cd);
 
 	  break;
 
@@ -370,7 +363,7 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 	case OP_CL_MINQUERY:
 	case OP_CL_ONCEQUERY:
 	  {
-	    register const uschar *classptr = tcode + 1;
+	    const uschar *classptr = tcode + 1;
 	    tcode++;
 	    switch (tcode[-1])
 	      {
@@ -404,30 +397,18 @@ set_boyer_moore (code, bmtable, length, caseless, cd)
 		      
 	      default:
 		length++;
-		for (c = 0; c < 255; c++)
-		  if (classptr[c >> 3] & (1 << (c & 7)))
-		    set_bmtable (bmtable, c, length, caseless, cd);
+		for (i = 0; i < 255; i++)
+		  if (classptr[i >> 3] & (1 << (i & 7)))
+		    set_bmtable (bmtable, i, length, caseless, cd);
 		break;
 	      }
 
 	    tcode += 32;
 	    break;		/* End of class handling */
 	  }
-
-	default:
-	  /* Undo the effect of a OP_TYPE... opcode */
-	  length = old_length;
-
-	  /* If a branch starts with a bracket, recurse to set the string
-	     from within them. That's all for this branch. */
-
-	  if ((int) *tcode >= OP_BRA)
-	      length = set_boyer_moore (tcode, bmtable,
-					length, caseless, cd);
-	  try_next = FALSE;
-	  break;
 	}			/* End of switch */
     }			/* End of try_next loop */
+  while (try_next && length < 255);
 
   return length;
 }
@@ -459,13 +440,13 @@ set_start_bits (code, start_bits, caseless, cd)
      BOOL caseless;
      compile_data *cd;
 {
-  register int c;
+  int i;
 
-/* This next statement and the later reference to dummy are here in order to
-   trick the optimizer of the IBM C compiler for OS/2 into generating correct
-   code. Apparently IBM isn't going to fix the problem, and we would rather not
-   disable optimization (in this module it actually makes a big difference, and
-   the pcre module can use all the optimization it can get). */
+  /* This next statement and the later reference to dummy are here in order to
+     trick the optimizer of the IBM C compiler for OS/2 into generating correct
+     code. Apparently IBM isn't going to fix the problem, and we would rather not
+     disable optimization (in this module it actually makes a big difference, and
+     the pcre module can use all the optimization it can get). */
 
   volatile int dummy;
 
@@ -474,7 +455,7 @@ set_start_bits (code, start_bits, caseless, cd)
       const uschar *tcode = code + 3;
       BOOL try_next = TRUE;
 
-      while (try_next)
+      do
 	{
 	  /* If a branch starts with a bracket or a positive lookahead assertion,
 	     recurse to set bits from within them. That's all for this branch. */
@@ -483,212 +464,215 @@ set_start_bits (code, start_bits, caseless, cd)
 	    {
 	      if (!set_start_bits (tcode, start_bits, caseless, cd))
 		return FALSE;
-	      try_next = FALSE;
+	      break;
 	    }
 
-	  else
-	    switch (*tcode)
-	      {
-	      default:
+	  switch (*tcode)
+	    {
+	    default:
+	      if ((int) *tcode >= OP_BRA || *tcode == OP_ASSERT)
+		abort ();
+	      else
 		return FALSE;
 
-		/* Skip over extended extraction bracket number */
+	      /* Skip over extended extraction bracket number */
 		
-	      case OP_BRANUMBER:
-		tcode += 3;
-		break;
-
-		/* Skip over lookbehind and negative lookahead assertions */
-
-	      case OP_ASSERT_NOT:
-	      case OP_ASSERTBACK:
-	      case OP_ASSERTBACK_NOT:
-		do
-		  tcode += (tcode[1] << 8) + tcode[2];
-		while (*tcode == OP_ALT);
-		tcode += 3;
-		break;
+	    case OP_BRANUMBER:
+	      tcode += 3;
+	      break;
 
+	      /* Skip over lookbehind and negative lookahead assertions */
+	      
+	    case OP_ASSERT_NOT:
+	    case OP_ASSERTBACK:
+	    case OP_ASSERTBACK_NOT:
+	      do
+		tcode += (tcode[1] << 8) + tcode[2];
+	      while (*tcode == OP_ALT);
+	      tcode += 3;
+	      break;
+	      
 		/* Skip over an option setting, changing the caseless flag */
+	      
+	    case OP_OPT:
+	      caseless = (tcode[1] & PCRE_CASELESS) != 0;
+	      tcode += 2;
+	      break;
+	      
+	      /* BRAZERO does the bracket, but carries on. */
+	      
+	    case OP_BRAZERO:
+	    case OP_BRAMINZERO:
+	      if (!set_start_bits (++tcode, start_bits, caseless, cd))
+		return FALSE;
+	      dummy = 1;
+	      do
+		tcode += (tcode[1] << 8) + tcode[2];
+	      while (*tcode == OP_ALT);
+	      tcode += 3;
+	      break;
+	      
+	      /* Single-char * or ? sets the bit and tries the next item */
+	      
+	    case OP_MAXSTAR:
+	    case OP_MINSTAR:
+	    case OP_ONCESTAR:
+	    case OP_MAXQUERY:
+	    case OP_MINQUERY:
+	    case OP_ONCEQUERY:
+	      set_bit (start_bits, tcode[1], caseless, cd);
+	      tcode += 2;
+	      break;
 
-	      case OP_OPT:
-		caseless = (tcode[1] & PCRE_CASELESS) != 0;
-		tcode += 2;
-		break;
-
-		/* BRAZERO does the bracket, but carries on. */
-
-	      case OP_BRAZERO:
-	      case OP_BRAMINZERO:
-		if (!set_start_bits (++tcode, start_bits, caseless, cd))
+	      /* Single-char upto sets the bit and tries the next */
+	      
+	    case OP_MAXUPTO:
+	    case OP_MINUPTO:
+	    case OP_ONCEUPTO:
+	      set_bit (start_bits, tcode[3], caseless, cd);
+	      tcode += 4;
+	      break;
+	      
+	      /* At least one single char sets the bit and stops */
+	      
+	    case OP_EXACT:	/* Fall through */
+	      tcode++;
+	      
+	    case OP_CHARS:	/* Fall through */
+	      tcode++;
+	      
+	    case OP_MAXPLUS:
+	    case OP_MINPLUS:
+	    case OP_ONCEPLUS:
+	      set_bit (start_bits, tcode[1], caseless, cd);
+	      try_next = FALSE;
+	      break;
+	      
+	      /* Single character type sets the bits and stops */
+	      
+	    case OP_TYPENOT:
+	      tcode++;
+	      try_next = FALSE;
+	    REPEATTYPENOT:
+	      if (!*tcode)
+		return FALSE;
+	      
+	      for (i = 0; i < 32; i++)
+		start_bits[i] |= ~cd->cbits[i + *tcode * 32];
+	      break;
+	      
+	    case OP_TYPE:
+	      tcode++;
+	      try_next = FALSE;
+	    REPEATTYPE:
+	      if (!*tcode)
 		  return FALSE;
-		dummy = 1;
-		do
-		  tcode += (tcode[1] << 8) + tcode[2];
-		while (*tcode == OP_ALT);
-		tcode += 3;
-		break;
-
-		/* Single-char * or ? sets the bit and tries the next item */
-
-	      case OP_MAXSTAR:
-	      case OP_MINSTAR:
-	      case OP_ONCESTAR:
-	      case OP_MAXQUERY:
-	      case OP_MINQUERY:
-	      case OP_ONCEQUERY:
-		set_bit (start_bits, tcode[1], caseless, cd);
+	      
+	      for (i = 0; i < 32; i++)
+		start_bits[i] |= cd->cbits[i + *tcode * 32];
+	      break;
+	      
+	      /* One or more character type fudges the pointer and restarts. */
+	      
+	    case OP_TYPEEXACT:
 		tcode += 2;
-		break;
-
-		/* Single-char upto sets the bit and tries the next */
-
-	      case OP_MAXUPTO:
-	      case OP_MINUPTO:
-	      case OP_ONCEUPTO:
-		set_bit (start_bits, tcode[3], caseless, cd);
-		tcode += 4;
-		break;
-
-		/* At least one single char sets the bit and stops */
-
-	      case OP_EXACT:	/* Fall through */
+		
+	    case OP_TYPE_MAXPLUS:
+	    case OP_TYPE_MINPLUS:
+	    case OP_TYPE_ONCEPLUS:
+	      tcode++;
+	      try_next = FALSE;
+	      goto REPEATTYPE;
+	      
+	      /* Zero or more repeats of character types set the bits and then
+		 try again. */
+	      
+	    case OP_TYPE_MAXUPTO:
+	    case OP_TYPE_MINUPTO:
+	    case OP_TYPE_ONCEUPTO:
+	      tcode += 2;	/* Fall through */
+	      
+	    case OP_TYPE_MAXSTAR:
+	    case OP_TYPE_MINSTAR:
+	    case OP_TYPE_ONCESTAR:
+	    case OP_TYPE_MAXQUERY:
+	    case OP_TYPE_MINQUERY:
+	    case OP_TYPE_ONCEQUERY:
+	      tcode++;
+	      goto REPEATTYPE;
+	      break;
+	      
+	      /* One or more character type fudges the pointer and restarts. */
+	      
+	    case OP_TYPENOTEXACT:
+	      tcode += 2;	/* Fall through */
+	      
+	    case OP_TYPENOT_MAXPLUS:
+	    case OP_TYPENOT_MINPLUS:
+	    case OP_TYPENOT_ONCEPLUS:
+	      try_next = FALSE;
+	      goto REPEATTYPENOT;
+	      
+	    case OP_TYPENOT_MAXUPTO:
+	    case OP_TYPENOT_MINUPTO:
+	    case OP_TYPENOT_ONCEUPTO:
+	      tcode += 2;	/* Fall through */
+	      
+	    case OP_TYPENOT_MAXSTAR:
+	    case OP_TYPENOT_MINSTAR:
+	    case OP_TYPENOT_ONCESTAR:
+	    case OP_TYPENOT_MAXQUERY:
+	    case OP_TYPENOT_MINQUERY:
+	    case OP_TYPENOT_ONCEQUERY:
+	      tcode++;
+	      goto REPEATTYPENOT;
+	      
+	      /* Character class: set the bits and either carry on or not,
+		 according to the repeat count. */
+	      
+	    case OP_CLASS:
+	    case OP_CL_MAXSTAR:
+	    case OP_CL_MINSTAR:
+	    case OP_CL_ONCESTAR:
+	    case OP_CL_MAXPLUS:
+	    case OP_CL_MINPLUS:
+	    case OP_CL_ONCEPLUS:
+	    case OP_CL_MAXQUERY:
+	    case OP_CL_MINQUERY:
+	    case OP_CL_ONCEQUERY:
+	    case OP_CL_MAXRANGE:
+	    case OP_CL_MINRANGE:
+	    case OP_CL_ONCERANGE:
+	      {
 		tcode++;
+		for (i = 0; i < 32; i++)
+		  start_bits[i] |= tcode[i];
+		tcode += 32;
+		switch (tcode[-33])
+		  {
+		  case OP_CL_MAXSTAR:
+		  case OP_CL_MINSTAR:
+		  case OP_CL_ONCESTAR:
+		  case OP_CL_MAXQUERY:
+		  case OP_CL_MINQUERY:
+		  case OP_CL_ONCEQUERY:
+		    break;
+		    
+		  case OP_CL_MAXRANGE:
+		  case OP_CL_MINRANGE:
+		  case OP_CL_ONCERANGE:
+		    if (((tcode[1] << 8) + tcode[2]) == 0)
+		      tcode += 4;
+		    else
+		      try_next = FALSE;
+		    break;
+		  }
+	      }
+	      break;		/* End of class handling */
 
-	      case OP_CHARS:	/* Fall through */
-		tcode++;
-
-	      case OP_MAXPLUS:
-	      case OP_MINPLUS:
-	      case OP_ONCEPLUS:
-		set_bit (start_bits, tcode[1], caseless, cd);
-		try_next = FALSE;
-		break;
-
-		/* Single character type sets the bits and stops */
-
-	      case OP_TYPENOT:
-		tcode++;
-		try_next = FALSE;
-	      REPEATTYPENOT:
-		if (!*tcode)
-		  return FALSE;
-
-		for (c = 0; c < 32; c++)
-		  start_bits[c] |= ~cd->cbits[c + *tcode * 32];
-		break;
-
-	      case OP_TYPE:
-		tcode++;
-		try_next = FALSE;
-	      REPEATTYPE:
-		if (!*tcode)
-		  return FALSE;
-
-		for (c = 0; c < 32; c++)
-		  start_bits[c] |= cd->cbits[c + *tcode * 32];
-		break;
-
-		/* One or more character type fudges the pointer and restarts. */
-
-	      case OP_TYPEEXACT:
-		tcode += 2;
-
-	      case OP_TYPE_MAXPLUS:
-	      case OP_TYPE_MINPLUS:
-	      case OP_TYPE_ONCEPLUS:
-		tcode++;
-		try_next = FALSE;
-		goto REPEATTYPE;
-
-		/* Zero or more repeats of character types set the bits and then
-		   try again. */
-
-	      case OP_TYPE_MAXUPTO:
-	      case OP_TYPE_MINUPTO:
-	      case OP_TYPE_ONCEUPTO:
-		tcode += 2;	/* Fall through */
-
-	      case OP_TYPE_MAXSTAR:
-	      case OP_TYPE_MINSTAR:
-	      case OP_TYPE_ONCESTAR:
-	      case OP_TYPE_MAXQUERY:
-	      case OP_TYPE_MINQUERY:
-	      case OP_TYPE_ONCEQUERY:
-		tcode++;
-		goto REPEATTYPE;
-		break;
-
-		/* One or more character type fudges the pointer and restarts. */
-
-	      case OP_TYPENOTEXACT:
-		tcode += 2;	/* Fall through */
-
-	      case OP_TYPENOT_MAXPLUS:
-	      case OP_TYPENOT_MINPLUS:
-	      case OP_TYPENOT_ONCEPLUS:
-		try_next = FALSE;
-		goto REPEATTYPENOT;
-
-	      case OP_TYPENOT_MAXUPTO:
-	      case OP_TYPENOT_MINUPTO:
-	      case OP_TYPENOT_ONCEUPTO:
-		tcode += 2;	/* Fall through */
-
-	      case OP_TYPENOT_MAXSTAR:
-	      case OP_TYPENOT_MINSTAR:
-	      case OP_TYPENOT_ONCESTAR:
-	      case OP_TYPENOT_MAXQUERY:
-	      case OP_TYPENOT_MINQUERY:
-	      case OP_TYPENOT_ONCEQUERY:
-		tcode++;
-		goto REPEATTYPENOT;
-
-		/* Character class: set the bits and either carry on or not,
-		   according to the repeat count. */
-
-	      case OP_CLASS:
-	      case OP_CL_MAXSTAR:
-	      case OP_CL_MINSTAR:
-	      case OP_CL_ONCESTAR:
-	      case OP_CL_MAXPLUS:
-	      case OP_CL_MINPLUS:
-	      case OP_CL_ONCEPLUS:
-	      case OP_CL_MAXQUERY:
-	      case OP_CL_MINQUERY:
-	      case OP_CL_ONCEQUERY:
-	      case OP_CL_MAXRANGE:
-	      case OP_CL_MINRANGE:
-	      case OP_CL_ONCERANGE:
-		{
-		  tcode++;
-		  for (c = 0; c < 32; c++)
-		    start_bits[c] |= tcode[c];
-		  tcode += 32;
-		  switch (tcode[-33])
-		    {
-		    case OP_CL_MAXSTAR:
-		    case OP_CL_MINSTAR:
-		    case OP_CL_ONCESTAR:
-		    case OP_CL_MAXQUERY:
-		    case OP_CL_MINQUERY:
-		    case OP_CL_ONCEQUERY:
-		      break;
-
-		    case OP_CL_MAXRANGE:
-		    case OP_CL_MINRANGE:
-		    case OP_CL_ONCERANGE:
-		      if (((tcode[1] << 8) + tcode[2]) == 0)
-			tcode += 4;
-		      else
-			try_next = FALSE;
-		      break;
-		    }
-		}
-		break;		/* End of class handling */
-
-	      }			/* End of switch */
+	    }			/* End of switch */
 	}			/* End of try_next loop */
+      while (try_next);
 
       code += (code[1] << 8) + code[2];		/* Advance to next branch */
     }
@@ -710,71 +694,64 @@ set_start_bits (code, start_bits, caseless, cd)
    Arguments:
    codeptr       points to the first opcode we're interested in,
                  on output it is rewritten with the end of the bracket
-   bracket_start points to an array of char_class'es that contain
+   bracket_start points to an array of bitsets that contain
                  the possible starting characters of the capturing
 		 subpatterns
-   bracket_end   points to an array of char_class'es that contain
+   bracket_end   points to an array of bitsets that contain
                  the possible ending characters of the capturing
 		 subpatterns
    caseless      the caseless flag
    cd            the block with char table pointers
-   end           points to where the function will store the character
-                 class for the end
+   p_start       points to where the function will store the character
+		 class starting the bracket
+   p_end         points to where the function will store the character
+                 class for the end, and will get the character class
+		 for the character immediately preceding the bracket
  */
 
 BOOL
-prune_bracket (prevptr, codeptr, bracket_start, 
-	       bracket_end, caseless, cd, end)
-     uschar       **prevptr, **codeptr;
+prune_bracket (codeptr, bracket_start, 
+	       bracket_end, caseless, cd, p_start, p_end)
+     uschar       **codeptr;
      BOOL         caseless;
-     char_class   *bracket_start, *bracket_end;
+     bitset       *bracket_start, *bracket_end, *p_start, *p_end;
      compile_data *cd;
-     char_class   end;
 {
-  char_class prev_class, curr_class, start;
-  uschar *previous = NULL, *current, *code = *codeptr;
-  int backref;
-  BOOL is_final_opcode = FALSE;
-  BOOL found_start = FALSE;
-  BOOL had_alt = FALSE;
-  BOOL can_be_empty;
-      
-  /* For extended extraction brackets (large number), we have to fish out the
-     number from a dummy opcode at the start. */
+  bitset all_alternatives_start, all_alternatives_end;
+  bitset prev_class, curr_class, start, end;
+  uschar *previous, *code = *codeptr;
+  BOOL is_final_opcode, found_start, end_same_as_start;
+  BOOL can_be_empty, bracket_can_be_empty = FALSE;
 
-  if (*code < OP_BRA)
-    {
-      backref = 0;
-      
-      /* Skip over the assertion at the beginning of a conditional */
-      if (*code == OP_COND && code[3] == OP_ASSERT)
-	code += (code[4] << 8) | code[5];
-    }
-  else
-    {
-      backref = *code - OP_BRA;
-      if (backref > EXTRACT_BASIC_MAX) 
-	backref = (code[4] << 8) | code[5];
-    }
+  /* Skip over the assertion at the beginning of a conditional */
+  if (*code == OP_COND && code[3] == OP_ASSERT)
+    code += (code[4] << 8) | code[5];
 
   code += 3;
-  current = code;
-  memset (curr_class, 0, 32);
+  memset (all_alternatives_start, 0, 32);
+  memset (all_alternatives_end, 0, 32);
+
+ restart:
+  if (p_end)
+    memcpy (curr_class, p_end, 32);
+  else
+    memset (curr_class, 0, 32);
+
+  memset (start, 0, 32);
   memset (end, ~0, 32);
+  previous = NULL;
+  found_start = FALSE;
 
   for (;;)
     {
-      int i, c;
-      /* Undefining CONSERVATIVE seems to work, and optimizes cases like
-	 b*c*b+ by leaving b* as is but pruning the c* backtracking path. */
+      int i, backref;
+      uschar *current = code;
 
-#ifdef CONSERVATIVE
-      memcpy (prev_class, end, 32);
-#else
       memcpy (prev_class, curr_class, 32);
-#endif
       memset (curr_class, 0, 32);
       can_be_empty = TRUE;
+      end_same_as_start = TRUE;
+      is_final_opcode = FALSE;
 
       switch (*code)
 	{
@@ -791,39 +768,50 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	case OP_KET_MINSTAR:
 	case OP_KET_MAXSTAR:
 	case OP_KET_ONCESTAR:
-	  code += 3;
-	  can_be_empty = FALSE;
 	  is_final_opcode = TRUE;
-	  memcpy (curr_class, prev_class, 32);
-
-	  if (backref > 0)
-	    {
-	      if (found_start)
-		memcpy (&bracket_start[backref], start, 32);
-	      else
-		memset (&bracket_start[backref], ~0, 32);
-	      if (had_alt)
-		memset (&bracket_end[backref], ~0, 32);
-	      else
-		memcpy (&bracket_end[backref], end, 32);
-	    }
-	  break;
 
 	case OP_ALT:
 	  code += 3;
-	  had_alt = TRUE;
-	  found_start = FALSE;
-	  can_be_empty = FALSE;
-	  memset (curr_class, ~0, 32);
-	  break;
+	  for (i = 0; i < 32; i++)
+	    {
+	      all_alternatives_start[i] |= start[i];
+	      all_alternatives_end[i] |= end[i];
+	    }
+
+	  if (!found_start)
+	    {
+	      bracket_can_be_empty = TRUE;
+	      memset (all_alternatives_start, ~0, 32);
+	      if (p_end)
+	        for (i = 0; i < 32; i++)
+		  all_alternatives_end[i] |= (*p_end)[i];
+	      else
+	        memset (all_alternatives_end, ~0, 32);
+	    }
+
+	  if (is_final_opcode)
+	    {
+	      if (p_start)
+	        memcpy (p_start, all_alternatives_start, 32);
+	      if (p_end)
+	        memcpy (p_end, all_alternatives_end, 32);
+
+	      *codeptr = code;
+	      return bracket_can_be_empty;
+	    }
+	  else
+	    goto restart;
 
 	  /* Skip over things that don't match chars */
 
 	case OP_DOLL:
 	case OP_EODN:
+	  end_same_as_start = FALSE;
+	  memcpy (end, prev_class, 32);
 	  memset (curr_class, ~0, 32);
 	  curr_class[13 / 8] ^= 1 << (13 % 8);
 
+	case OP_ANCHOR_MATCH:
 	case OP_SOD:
 	case OP_CIRC:
 	case OP_EOD:
@@ -832,19 +820,21 @@ prune_bracket (prevptr, codeptr, bracket_start,
 
 	case OP_NOT_WORD_BOUNDARY:
 	case OP_WORD_BOUNDARY:
+	  end_same_as_start = FALSE;
+	  memcpy (end, prev_class, 32);
 	  memset (curr_class, ~0, 32);
 	  code++;
 	  break;
 
 	case OP_BEG_WORD:
-	  for (c = 0; c < 32; c++)
-	    curr_class[c] = cd->cbits[c + cbit_word];
+	  for (i = 0; i < 32; i++)
+	    curr_class[i] = cd->cbits[i + cbit_word];
 	  code++;
 	  break;
 
 	case OP_END_WORD:
-	  for (c = 0; c < 32; c++)
-	    curr_class[c] = ~cd->cbits[c + cbit_word];
+	  for (i = 0; i < 32; i++)
+	    curr_class[i] = ~cd->cbits[i + cbit_word];
 	  code++;
 	  break;
 
@@ -860,8 +850,10 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  break;
 
 	case OP_ASSERT:
-	  prune_bracket (&current, &code, bracket_start, bracket_end,
-			 caseless, cd, &curr_class);
+	  prune_bracket (&code, bracket_start, bracket_end,
+			 caseless, cd, &curr_class, &end);
+	  memcpy (end, prev_class, 32);
+	  end_same_as_start = FALSE;
 	  break;
 
 	case OP_ASSERT_NOT:
@@ -880,16 +872,20 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  code += 2;
 	  break;
 
-	  /* BRAZERO does the bracket, but carries on. */
+	  /* BRAZERO does the bracket, but can be empty the classes. */
 
 	case OP_BRAZERO:
 	case OP_BRAMINZERO:
 	  code++;
-	  prune_bracket (&current, &code, bracket_start, bracket_end,
-			 caseless, cd, &curr_class);
+	  prune_bracket (&code, bracket_start, bracket_end,
+			 caseless, cd, &curr_class, &end);
+	  end_same_as_start = FALSE;
+	  memset (curr_class, ~0, 32);
+	  for (i = 0; i < 32; i++)
+	    end[i] |= prev_class[i];
 	  break;
 
-		/* Single character type sets the bits and stops */
+	  /* Single character type sets the bits */
 
 	case OP_TYPENOT:
 	  code++;
@@ -898,8 +894,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  if (!*code)
 	    memset (curr_class, ~0, 32);
 	  else
-	    for (c = 0; c < 32; c++)
-	      curr_class[c] = ~cd->cbits[c + *code * 32];
+	    for (i = 0; i < 32; i++)
+	      curr_class[i] = ~cd->cbits[i + *code * 32];
 
 	  code++;
 	  break;
@@ -911,8 +907,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  if (!*code)
 	    memset (curr_class, ~0, 32);
 	  else
-	    for (c = 0; c < 32; c++)
-	      curr_class[c] = cd->cbits[c + *code * 32];
+	    for (i = 0; i < 32; i++)
+	      curr_class[i] = cd->cbits[i + *code * 32];
 
 	  code++;
 	  break;
@@ -920,6 +916,7 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	case OP_CHARS:
 	  code++;
 	  can_be_empty = FALSE;
+	  end_same_as_start = FALSE;
 	  set_bit (curr_class, code[1], caseless, cd);
 	  code += 1 + (int) *code;
 	  memset (end, 0, 32);
@@ -937,8 +934,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  can_be_empty = FALSE;
 	REPEATNOT:
 	  set_bit (curr_class, *code++, caseless, cd);
-	  for (c = 0; c < 32; c++)
-	    curr_class[c] = ~curr_class[c];
+	  for (i = 0; i < 32; i++)
+	    curr_class[i] = ~curr_class[i];
 
 	  break;
 
@@ -954,8 +951,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  can_be_empty = FALSE;
 	  goto REPEATTYPE;
 
-		/* Zero or more repeats of character types set the bits and then
-		   try again. */
+	  /* Zero or more repeats of character types set the bits and then
+	     try again. */
 
 	case OP_TYPE_MAXUPTO:
 	case OP_TYPE_MINUPTO:
@@ -1010,8 +1007,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  can_be_empty = FALSE;
 	  goto REPEATCHARS;
 
-		/* Zero or more repeats of character types set the bits and then
-		   try again. */
+	  /* Zero or more repeats of character types set the bits and then
+	     try again. */
 
 	case OP_MAXUPTO:
 	case OP_MINUPTO:
@@ -1057,7 +1054,7 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  goto REPEATNOT;
 
 	  /* Character class: set the bits and either carry on or not,
-		   according to the repeat count. */
+	     according to the repeat count. */
 
 	case OP_CLASS:
 	case OP_CL_MAXSTAR:
@@ -1074,8 +1071,8 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	case OP_CL_ONCERANGE:
 	  {
 	    code++;
-	    for (c = 0; c < 32; c++)
-	      curr_class[c] |= code[c];
+	    for (i = 0; i < 32; i++)
+	      curr_class[i] |= code[i];
 	    code += 32;
 	    switch (code[-33])
 	      {
@@ -1104,7 +1101,7 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	  break;		/* End of class handling */
 
 	  /* Character class: set the bits and either carry on or not,
-		   according to the repeat count. */
+	     according to the repeat count. */
 
 	case OP_REF:
 	case OP_REF_MAXSTAR:
@@ -1123,7 +1120,7 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	    int number = (code[1] << 8) | code[2];
 	    code += 3;
 	    memcpy (curr_class, bracket_start[number], 32);
-	    memcpy (end, bracket_end[number], 32);
+	    end_same_as_start = FALSE;
 	    switch (code[-3])
 	      {
 	      case OP_REF_MAXSTAR:
@@ -1132,49 +1129,61 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	      case OP_REF_MAXQUERY:
 	      case OP_REF_MINQUERY:
 	      case OP_REF_ONCEQUERY:
+		for (i = 0; i < 32; i++)
+		  end[i] = bracket_end[number][i];
 		break;
-
+		
 	      case OP_REF_MAXRANGE:
 	      case OP_REF_MINRANGE:
 	      case OP_REF_ONCERANGE:
 		if (((code[1] << 8) + code[2]) == 0)
 		  code += 4;
-		break;
+
+	      case OP_REF_MAXPLUS:
+	      case OP_REF_MINPLUS:
+	      case OP_REF_ONCEPLUS:
+		memcpy (end, bracket_end[number], 32);
 	      }
 	  }
 	  break;		/* End of class handling */
 
 	default:
-	  if ((int) *code >= OP_BRA || *code == OP_ONCE || *code == OP_COND)
+	  if ((int) *code >= OP_BRA)
 	    {
-	      can_be_empty = !prune_bracket(&current, &code,
-					    bracket_start, bracket_end,
-					    caseless, cd, &curr_class);
-	      break;
+	      backref = *code - OP_BRA;
+	      if (backref > EXTRACT_BASIC_MAX)
+	        backref = (code[4] << 8) | code[5];
+	    }
+	  else if (*code == OP_ONCE || *code == OP_COND)
+	    backref = 0;
+	  else
+	    {
+	      printf ("What's this `%s'?\n", pcre_OP_names[*code]);
+	      abort();
 	    }
 
-	  printf ("What's this `%s'?\n", pcre_OP_names[*code]);
-	  abort();
+	  can_be_empty = prune_bracket(&code, bracket_start, bracket_end,
+				       caseless, cd, &curr_class, &end);
 
+	  memcpy (bracket_start[backref], curr_class, 32);
+	  memcpy (bracket_end[backref], end, 32);
+	  end_same_as_start = FALSE;
+	  break;
 	}			/* End of switch */
 
-      if (previous)
+      /* Check if we can prune the last path.  The check
+	 on can_be_empty avoids pruning b* in b*c*b+. */
+      if (previous && !can_be_empty)
 	{
-	  /* Check if we can prune the last path.  The check
-	     on can_be_empty avoids pruning b* in b*c*b+. */
-	  if (!can_be_empty)
-	    {
-	      for (i = 0; i < 32; i++)
-		if (curr_class[i] & prev_class[i])
-		  break;
+	  for (i = 0; i < 32; i++)
+	    if (curr_class[i] & prev_class[i])
+	      break;
 	      
-	      if (i == 32)
-		*previous = pruned[*previous];
-	    }
+	  if (i == 32)
+	    *previous = pruned[*previous];
 	}
 
-      if (*current != OP_CHARS && 
-	  (*current <= OP_REF || *current >= OP_REF_ONCERANGE))
+      if (end_same_as_start)
 	{
 	  if (can_be_empty)
 	    {
@@ -1195,16 +1204,7 @@ prune_bracket (prevptr, codeptr, bracket_start,
 	    start[i] |= curr_class[i];
 	}
 
-      if (is_final_opcode)
-	{
-	  *prevptr = current;
-	  *codeptr = code;
-	  memcpy (end, curr_class, 32);
-	  return found_start;
-	}
-
-      previous = (pruned[*current] == *current) ? NULL : current;
-      current = code;
+      previous = (*current < sizeof (pruned) && pruned[*current]) ? current : NULL;
     }
 }
 
@@ -1230,17 +1230,17 @@ prune_backtracking_paths (re, caseless, cd)
      BOOL          caseless;
      compile_data  *cd;
 {
-  char_class curr_class;
-  uschar *code = re->code, *current;
+  uschar *code = re->code;
 
-  char_class *bracket_start = (char_class *)
-    alloca(sizeof(char_class) * (1 + re->top_bracket));
+  /* The first entry is used as scratch in both bitsets.  */
+  bitset *bracket_start = (bitset *)
+    alloca(sizeof(bitset) * (1 + re->top_bracket));
 
-  char_class *bracket_end = (char_class *)
-    alloca(sizeof(char_class) * (1 + re->top_bracket));
+  bitset *bracket_end = (bitset *)
+    alloca(sizeof(bitset) * (1 + re->top_bracket));
 
-  prune_bracket (&current, &code, bracket_start, bracket_end,
-		 caseless, cd, &curr_class);
+  prune_bracket (&code, bracket_start, bracket_end,
+		 caseless, cd, NULL, NULL);
 }
 
 
@@ -1290,7 +1290,7 @@ pcre_study (re, options, errorptr)
     }
 
 
-/* Set the character tables in the block which is passed around */
+  /* Set the character tables in the block which is passed around */
 
   compile_block.lcc = re->tables + lcc_offset;
   compile_block.fcc = re->tables + fcc_offset;
@@ -1301,14 +1301,14 @@ pcre_study (re, options, errorptr)
   if (!(options & PCRE_STUDY_NO_PRUNE))
     prune_backtracking_paths(re, caseless, &compile_block);
 
-/* For an anchored pattern, or a multiline pattern that matches only at
-   "line starts", no further processing at present. */
+  /* For an anchored pattern, or a multiline pattern that matches only at
+     "line starts", no further processing at present. */
 
   if ((re->options & (PCRE_ANCHORED | PCRE_STARTLINE)) != 0
       || (options & PCRE_STUDY_NO_START))
     return NULL;
 
-/* See if we can find a fixed length initial string for the pattern. */
+  /* See if we can find a fixed length initial string for the pattern. */
 
   memset ((char *) &myextra, 0, sizeof (myextra));
 
