@@ -168,7 +168,7 @@ main (int argc, char **argv)
 	  printf
 	    ("Usage:   pcretest [options...] [<input> [<output>]]\n");
 	  printf ("  -d   --debug          debug: show compiled code; implies -i\n");
-	  printf ("  -s   --showinfo       show information about compiled pattern\n");
+	  printf ("  -i   --showinfo       show information about compiled pattern\n");
 	  printf ("  -o N --offsets-size=N set size of offsets vector to <n>\n");
 	  printf ("  -p   --regexec        use POSIX interface\n");
 	  printf ("  -P   --posix          use POSIX regular expressions\n");
@@ -255,7 +255,7 @@ main (int argc, char **argv)
       p = buffer;
       while (isspace (*p))
 	p++;
-      if (*p == 0)
+      if (*p == 0 || *p == '#')
 	continue;
 
       /* Get the delimiter and seek the end of the pattern; if is isn't
